package custom;
#
#  Package:      custom.pm
#
#  Description:  The OS specific routines for changing the system.
#    The routines in this package assume the XML has already been
#    changed.  They attempt to bring integrity back to the system
#    by adding/removing users, changing network settings, restarting
#    services, writing out to config files, etc.
#
#  History:
#
use strict;

use CGI;
use XML::DOM;
use XML::QL;
use Quota;
use Time::localtime;

use config;
use general;
use datastore;
use message;
use error;

# A sub-routine to update the system date and time
sub updateDateTime {

    my $mode = shift || return(0);
    my $datetime = shift || return(0);
    my $timezone = shift || "";
    my $timeserver = shift || "";
    my $frequency = shift || "No";

    my %system = config::system();

    # Update the time zone file
    if ($timezone ne "") {
	# Copy the right time zone file over
	`/bin/cp $timezone /etc/localtime`;	
    }

    # Use a manual update
    my $command = "";
    my $result = "";
    my $errors = "";

    # Update via a time server and setup time synch
    if ($mode eq "ntp") {
	
	eval {
	    
	    # Set error handling
	    local $SIG{ALRM} = sub {
		my $errormsg = error::throwError("NTP_TIMEOUT"); 
		print "NTP_TIMEOUT";
		warn("$errormsg");
		die("$errormsg");
	    };
	    
	    # Set timeout
	    alarm(($system{'ntptimeout'}));

	    open(STDERR, ">/tmp/ntpdateresults.tmp");
	    $| = 1;
	    $command = "/usr/sbin/ntpdate -t1 $timeserver";
	    print STDERR "Making NTPDATE call = $command\n";
	    $result = `$command`;

	    if (open (ERRORS, "</tmp/ntpdateresults.tmp")) {
		
		my @errorlines = <ERRORS>;

		$errors = join("\n",@errorlines);
		
		close(ERRORS);
	    }

	};

#	print STDERR "Results = $result\n";

	# Check to see if ntpdate timed out
	if ($errors =~ m/no server suitable/) {
	    print "NTP_NOSERVICE";
	    exit(0);
	}
	elsif ($errors =~ m/can\'t find host/) {
	    print "NTP_NOHOST";
	    exit(0);
	}
	# Print the date/time that was returned
	else {
	    my @values = split("ntpdate", $result);
	    print "$values[0]\n";
	}

    }
    # Update manually
    else {
	$command = "$system{'root'}/system/bin/date --set='$datetime'";
	$result = `$command`;	
    }

    # Setup time synchronization to occur (or not occur) on a routine basis
    # Grab current crontab
    $command = "/usr/bin/crontab -l";
    $result = `$command`;

    $result =~ s/\"/\\\"/g;

#    print STDERR "CRONTAB = $result\n";
    
    my @lines = split('\n', $result);
    my @newlines = ();
    
    my $numOfLines = scalar(@lines);
    
    for (my $i=3; $i<$numOfLines; $i++) {
	
	# Remove the time synch lines
	if ($lines[$i] =~ /# Compaq Configurator/) {
	    next;
	}
	elsif ($lines[$i] =~ m/\/system\/bin\/ntpdate/) {
	    next;
	}
	# Get rid of blank lines
	elsif ($lines[$i] eq "\n") {
	    next;
	}
	# Preserve existing entries
	else {
	    push(@newlines, $lines[$i]);
	}
    }

    # Add time synch lines
    if ($frequency ne "No") {
	push (@newlines, "\n# Compaq Configurator - Time Synchronization");
    }
    
    if ($frequency eq "hour") {
	push (@newlines, "0 * * * * $system{'root'}/system/bin/ntpdate -s -t 3 $timeserver");
    }
    elsif ($frequency eq "day") {
	push (@newlines, "0 0 * * * $system{'root'}/system/bin/ntpdate -s -t 3 $timeserver");
    }
    elsif ($frequency eq "week") {
	push (@newlines, "0 0 * * 0 $system{'root'}/system/bin/ntpdate -s -t 3 $timeserver");
    }
    elsif ($frequency eq "month") {
	push (@newlines, "0 0 1 * * $system{'root'}/system/bin/ntpdate -s -t 3 $timeserver");
    }
    
    $command = "echo \"";
    
    foreach my $line (@newlines) {
	$command .= "$line\n";
    }
    
    $command .="\" | /usr/bin/crontab -";

    $result = `$command`;

    return(1);

}

# A sub-routine to update the root password
sub updateRoot {

    my $password = shift || return;

    # Encrypt password
    $password = crypt($password, substr($password,0,2));

    my $command = "/usr/sbin/usermod -p $password root";
    print "$command\n";
    print `$command`;

    return(1);

}

# A sub-routine to update the routing table
sub updateRouting {

    my %system = config::system();

    my $configFile = "/etc/sysconfig/static-routes";

    if (open(FILE, ">$configFile")) {

	my $parser = new XML::DOM::Parser;
	my $doc = $parser->parsefile("$system{'datastore'}/network.xml");

	my $dfgw = eval { $doc->getElementsByTagName("DefaultGateway")->item(0)->getFirstChild->getData(); } || "";

	my $result = "";

	# Step 2: Grab the routes
	foreach my $routeNode ($doc->getElementsByTagName("Route")) {
	    
	    my $type = eval { $routeNode->getElementsByTagName("RouteType")->item(0)->getFirstChild->getData(); } || "";
	    my $dest = eval { $routeNode->getElementsByTagName("Destination")->item(0)->getFirstChild->getData(); } || ""; 
	    my $gw = eval { $routeNode->getElementsByTagName("Gateway")->item(0)->getFirstChild->getData(); } || "";
	    my $netmask = eval { $routeNode->getElementsByTagName("NetMask")->item(0)->getFirstChild->getData(); } || "";
	    my $if = eval { $routeNode->getElementsByTagName("Interface")->item(0)->getFirstChild->getData(); } || "";
	    $if = lc $if;
	    
	    $result .= "$type<TD>$dest<TD>$gw<TD>$netmask<TD>$if<TD><TR>";
	}

	$doc->dispose();
	
	my @rows = split('<TR>', $result);
	
	my $date = `/bin/date`;
	my $message = "# File Modified by Compaq Configurator - $date";
	
	print FILE "$message\n";

	my $gateway = "";
	my $command = "";

    	# Cycle the network cards
    	my $command = "/sbin/ifconfig eth0 down";
    	`$command`;	
    	$command = "/sbin/ifconfig eth1 down";
    	`$command`;	
	
    	$command = "/sbin/ifconfig eth0 up";
    	`$command`;	
    	$command = "/sbin/ifconfig eth1 up";
    	`$command`;

#  	print STDERR "Cycled network cards!\n";

	# Delete each route
	my $command = "/sbin/route -n";
	my $result = `$command`;

	my @currroutes = split('\n', $result);
	shift(@currroutes);

	foreach my $delroute (@currroutes) {

	    my @values = split(' ', $delroute);
	    $command = "/sbin/route del $values[0] gw $values[1]";
#	    print STDERR " DELETE ROUTE = $command\n";
	    $result = `$command`;
	}

	# Process each route
	foreach my $row (@rows) {

	    my ($type, $dest, $gw, $netmask, $if) = split('<TD>', $row);

	    if ($gateway eq "" && $gw ne "") {
		$gateway = $gw;
	    }

	    print FILE "$if $type $dest netmask $netmask gw $gw\n";

	    if ($type eq "net") {
		$command = "/sbin/route -n add -net $dest netmask $netmask gw $gw dev $if";
		$result = `$command`;
	    }
	    else {
		$command = "/sbin/route -n add -host $dest gw $gw dev $if";
		$result = `$command`;
	    }

#	    print STDERR "ROUTING COMMAND = $command\n";


	}

	# Add the default gateway back into the routing table
	$dfgw =~ s/ //g;
	$command = "/sbin/route -n add default gw $dfgw";
	$result = `$command`;

	print STDERR "Added default gateway; command = $command  ($dfgw)!\n";

	close (FILE);
    
    }
}

# A sub-routine to update the services configuration
sub updateServices {

    my %system = config::system();

    my $configFile = "/etc/sysconfig/ipchains";
    my $rules = ":input ACCEPT\n:forward ACCEPT\n:output ACCEPT\n";

    my $proftp_file = "
# This is a basic ProFTPD configuration file (rename it to 
# 'proftpd.conf' for actual use.  It establishes a single server
# and a single anonymous login.  It assumes that you have a user/group
# \"nobody\" and \"ftp\" for normal operation and anon.

ServerName			\"ProFTPD Default Installation\"
ServerType	standalone
DefaultServer			on

# Port 21 is the standard FTP port.
Port				21
# Umask 022 is a good standard umask to prevent new dirs and files
# from being group and world writable.
Umask				022

# To prevent DoS attacks, set the maximum number of child processes
# to 30.  If you need to allow more than 30 concurrent connections
# at once, simply increase this value.  Note that this ONLY works
# in standalone mode, in inetd mode you should use an inetd server
# that allows you to limit maximum number of processes per service
# (such as xinetd)
MaxInstances			30

# Set the user and group that the server normally runs at.
User				nobody
Group				nobody

# Normally, we want files to be overwriteable.
<Directory /*>
  AllowOverwrite		on
</Directory>

";

    # Step 1:  Read in the service configuration for each network card
    # Query the XML for our services
    my $query = 'WHERE
      <Interface if="$if">
         <Enabled>$enabled</Enabled>
         <IP>$ip</IP>
         <NetMask>$netmask</NetMask>
         <Network>$net</Network>
         <Broadcast>$bcast</Broadcast>
         <Telnet>$telnet</Telnet>
         <FTP>$ftp</FTP>
         <AnonFTP>$anonftp</AnonFTP>
         <SSH>$ssh</SSH>
         <SMTP>$smtp</SMTP>
         <SNMP>$snmp</SNMP>
         <POP3>$pop3</POP3>
         <IMAP>$imap</IMAP>
         <NFS>$nfs</NFS>
         <CompaqConfigurator>$cpqcfg</CompaqConfigurator>
      </Interface>
      IN "file:' . $system{'datastore'} . '/network.xml"
      CONSTRUCT $enabled<TD>$ip<TD>$if<TD>$telnet<TD>$ftp<TD>$anonftp<TD>$smtp<TD>$snmp<TD>$ssh<TD>$pop3<TD>$imap<TD>$nfs<TD>$cpqcfg<TR>';

    my $result = XML::QL->query($query);

#    print STDERR "Result = $result\n";

    my @rows = split("<TR>", $result);

    # Step 2:  Execute IPChains command for each service
    foreach my $row (@rows) {

	my ($enabled, $ip, $if, $telnet, $ftp, $anonftp, $smtp, $snmp, $ssh, $pop3, $imap, $nfs, $cpqcfg) = split("<TD>", $row);

#	print STDERR "result = $result\nANONYMOUS FTP is $anonftp\n";

	my $command = "";

	# Enable Telnet
	$command = "/sbin/ipchains -D input -d $ip --destination-port 23 -j REJECT -p tcp";
	`$command`;
	$command = "/sbin/ipchains -D output -s $ip --source-port 23 -j REJECT -p tcp";
	`$command`;

#	print STDERR "Command = $command\n";

	if ($telnet ne "Yes") {

 	    $rules .= "-A input -s 0.0.0.0/0.0.0.0 23:23 -d $ip -j REJECT -p tcp
-A output -s $ip 23:23 -d 0.0.0.0/0.0.0.0 -j REJECT -p tcp\n";
	    # Disable Telnet
	    $command = "/sbin/ipchains -A input -d $ip --destination-port 23 -j REJECT -p tcp";
	    `$command`;
	    $command = "/sbin/ipchains -A output -s $ip --source-port 23 -j REJECT -p tcp";
	    `$command`;

#	    print STDERR "Command = $command\n";

	}


	# Enable FTP
	$command = "/sbin/ipchains -D input -d $ip --destination-port 21 -j REJECT -p tcp";
	`$command`;
#	print STDERR "$command\n";
	$command = "/sbin/ipchains -D output -s $ip --source-port 21 -j REJECT -p tcp";
	`$command`;
#	print STDERR "$command\n";

	if ($ftp ne "Yes") {

 	    $rules .= "-A input -s 0.0.0.0/0.0.0.0 21:21 -d $ip -j REJECT -p tcp
-A output -s $ip 21:21 -d 0.0.0.0/0.0.0.0 -j REJECT -p tcp\n";


	    # Disable FTP
	    $command = "/sbin/ipchains -A input -d $ip --destination-port 21 -j REJECT -p tcp";
	    `$command`;
#	    print STDERR "$command\n";
	    $command = "/sbin/ipchains -A output -s $ip --source-port 21 -j REJECT -p tcp";
	    `$command`;
#	    print STDERR "$command\n";


	}

	# Enable/Disable Anonymous FTP
	if ($anonftp eq "Yes" && $enabled eq "Yes") {

	    # Make sure the ftp user has a valid shell
	    `/usr/sbin/usermod -s /bin/bash ftp`;

	    $proftp_file .= "<VirtualHost $ip>

     # Normally, we want files to be overwriteable.
    <Directory /*>
      AllowOverwrite		on
    </Directory>

    <Anonymous ~ftp>
  User                          ftp
  Group                         ftp
  # We want clients to be able to login with \"anonymous\" as well as \"ftp\"
  UserAlias                     anonymous ftp

  # Limit the maximum number of anonymous logins
  MaxClients                    10

  # We want 'welcome.msg' displayed at login, and '.message' displayed
  # in each newly chdired directory.
  DisplayLogin                  welcome.msg
  DisplayFirstChdir             .message

  # Limit WRITE everywhere in the anonymous chroot
  <Limit WRITE>
    DenyAll
  </Limit>

</Anonymous>            
</VirtualHost>\n";
	}

	# Enable SSH
	$command = "/sbin/ipchains -D input -d $ip --destination-port 22 -j REJECT -p tcp";
	`$command`;
#	print STDERR "$command\n";
	$command = "/sbin/ipchains -D output -s $ip --source-port 22 -j REJECT -p tcp";
	`$command`;
#	print STDERR "$command\n";
	$command = "/sbin/ipchains -D input -d $ip --destination-port 22 -j REJECT -p udp";
	`$command`;
#	print STDERR "$command\n";
	$command = "/sbin/ipchains -D output -s $ip --source-port 22 -j REJECT -p udp";
	`$command`;
#	print STDERR "$command\n";


	if ($ssh ne "Yes") {

 	    $rules .= "-A input -s 0.0.0.0/0.0.0.0 22:22 -d $ip -j REJECT -p tcp
-A output -s $ip 22:22 -d 0.0.0.0/0.0.0.0 -j REJECT -p tcp
-A input -s 0.0.0.0/0.0.0.0 22:22 -d $ip -j REJECT -p udp
-A output -s $ip 22:22 -d 0.0.0.0/0.0.0.0 -j REJECT -p udp\n";

	    # Disable SSH
	    $command = "/sbin/ipchains -A input -d $ip --destination-port 22 -j REJECT -p tcp";
	    `$command`;
#	    print STDERR "$command\n";
	    $command = "/sbin/ipchains -A output -s $ip --source-port 22 -j REJECT -p tcp";
	    `$command`;
#	    print STDERR "$command\n";
	    $command = "/sbin/ipchains -A input -d $ip --destination-port 22 -j REJECT -p udp";
	    `$command`;
#	    print STDERR "$command\n";
	    $command = "/sbin/ipchains -A output -s $ip --source-port 22 -j REJECT -p udp";
	    `$command`;
#	    print STDERR "$command\n";


	}

	# Enable SMTP
	$command = "/sbin/ipchains -D input -d $ip --destination-port 25 -j REJECT -p tcp";
	`$command`;
#	print STDERR "$command\n";
	$command = "/sbin/ipchains -D output -s $ip --source-port 25 -j REJECT -p tcp";
	`$command`;
#	print STDERR "$command\n";
	$command = "/sbin/ipchains -D input -d $ip --destination-port 25 -j REJECT -p udp";
	`$command`;
#	print STDERR "$command\n";
	$command = "/sbin/ipchains -D output -s $ip --source-port 25 -j REJECT -p udp";
	`$command`;
#	print STDERR "$command\n";


	if ($smtp ne "Yes") {

 	    $rules .= "-A input -s 0.0.0.0/0.0.0.0 25:25 -d $ip -j REJECT -p tcp
-A output -s 0.0.0.0/0.0.0.0 25:25 -d $ip -j REJECT -p udp
-A input -s 0.0.0.0/0.0.0.0 25:25 -d $ip -j REJECT -p tcp
-A output -s 0.0.0.0/0.0.0.0 25:25 -d $ip -j REJECT -p udp\n";

	    # Disable SMTP
	    $command = "/sbin/ipchains -A input -d $ip --destination-port 25 -j REJECT -p tcp";
	    `$command`;
#	    print STDERR "$command\n";
	    $command = "/sbin/ipchains -A output -d $ip --source-port 25 -j REJECT -p tcp";
	    `$command`;
#	    print STDERR "$command\n";
	    $command = "/sbin/ipchains -A input -d $ip --destination-port 25 -j REJECT -p udp";
	    `$command`;
#	    print STDERR "$command\n";
	    $command = "/sbin/ipchains -A output -d $ip --source-port 25 -j REJECT -p udp";
	    `$command`;
#	    print STDERR "$command\n";

	}

	# Enable SNMP
	$command = "/sbin/ipchains -D input -d $ip --destination-port 161 -j REJECT -p udp";
        `$command`;
#       print STDERR "$command\n";
	$command = "/sbin/ipchains -D output -s $ip --source-port 161 -j REJECT -p udp";
	`$command`;
#       print STDERR "$command\n";
	$command = "/sbin/ipchains -D input -d $ip --destination-port 162 -j REJECT -p udp";
	`$command`;
#       print STDERR "$command\n";
	$command = "/sbin/ipchains -D output -s $ip --source-port 162 -j REJECT -p udp";
	`$command`;
#       print STDERR "$command\n";

	if ($snmp ne "Yes") {

 	    $rules .= "-A input -s 0.0.0.0/0.0.0.0 161:161 -d $ip -j REJECT -p udp
-A output -s $ip 161:161 -d 0.0.0.0/0.0.0.0 -j REJECT -p udp
-A input -s 0.0.0.0/0.0.0.0 162:162 -d $ip -j REJECT -p udp
-A output -s $ip 162:162 -d 0.0.0.0/0.0.0.0 -j REJECT -p udp\n";


	    # Disable SNMP
	    
	    $command = "/sbin/ipchains -A input -d $ip --destination-port 161 -j REJECT -p udp";
	    `$command`;
#           print STDERR "$command\n";
	    $command = "/sbin/ipchains -A output -s $ip --source-port 161 -j REJECT -p udp";
	    `$command`;
#           print STDERR "$command\n";
	    $command = "/sbin/ipchains -A input -d $ip --destination-port 162 -j REJECT -p udp";
	    `$command`;
#           print STDERR "$command\n";
	    $command = "/sbin/ipchains -A output -s $ip --source-port 162 -j REJECT -p udp";
	    `$command`;
#           print STDERR "$command\n";

	}

	# Enable POP3
	$command = "/sbin/ipchains -D input -d $ip --destination-port 110 -j REJECT -p tcp";
	`$command`;
#	print STDERR "$command\n";
	$command = "/sbin/ipchains -D output -s $ip --source-port 110 -j REJECT -p tcp";
	`$command`;
#	print STDERR "$command\n";
	$command = "/sbin/ipchains -D input -d $ip --destination-port 110 -j REJECT -p udp";
	`$command`;
#	print STDERR "$command\n";
	$command = "/sbin/ipchains -D output -s $ip --source-port 110 -j REJECT -p udp";
	`$command`;
#	print STDERR "$command\n";


	if ($pop3 ne "Yes") {

 	    $rules .= "-A input -s 0.0.0.0/0.0.0.0 110:110 -d $ip -j REJECT -p tcp
-A output -s $ip 110:110 -d 0.0.0.0/0.0.0.0 -j REJECT -p tcp
-A input -s 0.0.0.0/0.0.0.0 110:110 -d $ip -j REJECT -p udp
-A output -s $ip 110:110 -d 0.0.0.0/0.0.0.0 -j REJECT -p udp\n";

	    # Disable POP3
	    $command = "/sbin/ipchains -A input -d $ip --destination-port 110 -j REJECT -p tcp";
	    `$command`;
#	    print STDERR "$command\n";
	    $command = "/sbin/ipchains -A output -s $ip --source-port 110 -j REJECT -p tcp";
	    `$command`;
#	    print STDERR "$command\n";
	    $command = "/sbin/ipchains -A input -d $ip --destination-port 110 -j REJECT -p udp";
	    `$command`;
#	    print STDERR "$command\n";
	    $command = "/sbin/ipchains -A output -s $ip --source-port 110 -j REJECT -p udp";
	    `$command`;
#	    print STDERR "$command\n";


	}
        
	# Enable IMAP
        
	
	$command = "/sbin/ipchains -D input -d $ip --destination-port 143 -j REJECT -p tcp";
	`$command`;
#	print STDERR "$command\n";
	$command = "/sbin/ipchains -D output -s $ip --source-port 143 -j REJECT -p tcp";
	`$command`;
#	print STDERR "$command\n";
	$command = "/sbin/ipchains -D input -d $ip --destination-port 143 -j REJECT -p udp";
	`$command`;
#	print STDERR "$command\n";
	$command = "/sbin/ipchains -D output -s $ip --source-port 143 -j REJECT -p udp";
	`$command`;
#	print STDERR "$command\n";

	$command = "/sbin/ipchains -D input -d $ip --destination-port 220 -j REJECT -p tcp";
	`$command`;
#	print STDERR "$command\n";
	$command = "/sbin/ipchains -D output -s $ip --source-port 220 -j REJECT -p tcp";
	`$command`;
#	print STDERR "$command\n";
	$command = "/sbin/ipchains -D input -d $ip --destination-port 220 -j REJECT -p udp";
	`$command`;
#	print STDERR "$command\n";
	$command = "/sbin/ipchains -D output -s $ip --source-port 220 -j REJECT -p udp";
	`$command`;
#	print STDERR "$command\n";


	$command = "/sbin/ipchains -D input -d $ip --destination-port 993 -j REJECT -p tcp";
	`$command`;
#	print STDERR "$command\n";
	$command = "/sbin/ipchains -D output -s $ip --source-port 993 -j REJECT -p tcp";
	`$command`;
#	print STDERR "$command\n";
	$command = "/sbin/ipchains -D input -d $ip --destination-port 993 -j REJECT -p udp";
	`$command`;
#	print STDERR "$command\n";
	$command = "/sbin/ipchains -D output -s $ip --source-port 993 -j REJECT -p udp";
	`$command`;
#	print STDERR "$command\n";

	if ($imap ne "Yes") {

 	    $rules .= "-A input -s 0.0.0.0/0.0.0.0 143:143 -d $ip -j REJECT -p tcp
-A output -s $ip 143:143 -d 0.0.0.0/0.0.0.0 -j REJECT -p tcp
-A input -s 0.0.0.0/0.0.0.0 143:143 -d $ip -j REJECT -p udp
-A output -s $ip 143:143 -d 0.0.0.0/0.0.0.0 -j REJECT -p udp
-A input -s 0.0.0.0/0.0.0.0 220:220 -d $ip -j REJECT -p tcp
-A output -s $ip 220:220 -d 0.0.0.0/0.0.0.0 -j REJECT -p tcp
-A input -s 0.0.0.0/0.0.0.0 220:220 -d $ip -j REJECT -p udp
-A output -s $ip 220:220 -d 0.0.0.0/0.0.0.0 -j REJECT -p udp
-A input -s 0.0.0.0/0.0.0.0 993:993 -d $ip -j REJECT -p tcp
-A output -s $ip 993:993 -d 0.0.0.0/0.0.0.0 -j REJECT -p tcp
-A input -s 0.0.0.0/0.0.0.0 993:993 -d $ip -j REJECT -p udp
-A output -s $ip 993:993 -d 0.0.0.0/0.0.0.0 -j REJECT -p udp\n";

	    # Disable IMAP
	    $command = "/sbin/ipchains -A input -d $ip --destination-port 143 -j REJECT -p tcp";
	    `$command`;
#	    print STDERR "$command\n";
	    $command = "/sbin/ipchains -A output -s $ip --source-port 143 -j REJECT -p tcp";
	    `$command`;
#	    print STDERR "$command\n";
	    $command = "/sbin/ipchains -A input -d $ip --destination-port 143 -j REJECT -p udp";
	    `$command`;
#	    print STDERR "$command\n";
	    $command = "/sbin/ipchains -A output -s $ip --source-port 143 -j REJECT -p udp";
	    `$command`;
#	    print STDERR "$command\n";
	    $command = "/sbin/ipchains -A input -d $ip --destination-port 220 -j REJECT -p tcp";
	    `$command`;
#	    print STDERR "$command\n";
	    $command = "/sbin/ipchains -A output -s $ip --source-port 220 -j REJECT -p tcp";
	    `$command`;
#	    print STDERR "$command\n";
	    $command = "/sbin/ipchains -A input -d $ip --destination-port 220 -j REJECT -p udp";
	    `$command`;
#	    print STDERR "$command\n";
	    $command = "/sbin/ipchains -A output -s $ip --source-port 220 -j REJECT -p udp";
	    `$command`;
#	    print STDERR "$command\n";
	    $command = "/sbin/ipchains -A input -d $ip --destination-port 993  -j REJECT -p tcp";
	    `$command`;
#	    print STDERR "$command\n";
	    $command = "/sbin/ipchains -A output -s $ip --source-port 993 -j REJECT -p tcp";
	    `$command`;
#	    print STDERR "$command\n";
	    $command = "/sbin/ipchains -A input -d $ip --destination-port 993 -j REJECT -p udp";
	    `$command`;
#	    print STDERR "$command\n";
	    $command = "/sbin/ipchains -A output -s $ip --source-port 993 -j REJECT -p udp";
	    `$command`;
#	    print STDERR "$command\n";
	}
        
	    
       
	# Enable NFS
	$command = "/sbin/ipchains -D input -d $ip --destination-port 2049 -j REJECT -p tcp";
	`$command`;
	$command = "/sbin/ipchains -D output -s $ip --source-port 2049 -j REJECT -p tcp";
	`$command`;

#	print STDERR "Command = $command\n";

	if ($nfs ne "Yes") {

 	    $rules .= "-A input -s 0.0.0.0/0.0.0.0 2049:2049 -d $ip -j REJECT -p tcp
-A output -s $ip 2049:2049 -d 0.0.0.0/0.0.0.0 -j REJECT -p tcp\n";


	    # Disable NFS
	    $command = "/sbin/ipchains -A input -d $ip --destination-port 2049 -j REJECT -p tcp";
	    `$command`;
	    $command = "/sbin/ipchains -A output -s $ip --source-port 2049 -j REJECT -p tcp";
	    `$command`;

#	    print STDERR "Command = $command\n";

	}

	# Enable Compaq Configurator
	$command = "/sbin/ipchains -D input -d $ip --destination-port 3201 -j REJECT -p tcp";
	`$command`;
	$command = "/sbin/ipchains -D output -s $ip --source-port 3201 -j REJECT -p tcp";
	`$command`;
	$command = "/sbin/ipchains -D input -d $ip --destination-port 3201 -j REJECT -p udp";
	`$command`;
	$command = "/sbin/ipchains -D output -s $ip --source-port 3201 -j REJECT -p udp";
	`$command`;

#	print STDERR "Command = $command\n";

	if ($cpqcfg ne "Yes") {

 	    $rules .= "-A input -s 0.0.0.0/0.0.0.0 3201:3201 -d $ip -j REJECT -p tcp
-A output -s $ip 3201:3201 -d 0.0.0.0/0.0.0.0 -j REJECT -p tcp
-A input -s 0.0.0.0/0.0.0.0 3201:3201 -d $ip -j REJECT -p udp
-A output -s $ip 3201:3201 -d 0.0.0.0/0.0.0.0 -j REJECT -p udp\n";


	    # Disable Compaq Configurator
	    $command = "/sbin/ipchains -A input -d $ip --destination-port 3201 -j REJECT -p tcp";
	    `$command`;
	    $command = "/sbin/ipchains -A output -s $ip --source-port 3201 -j REJECT -p tcp";
	    `$command`;
#	    print STDERR "Command = $command\n";
	    $command = "/sbin/ipchains -A input -d $ip --destination-port 3201 -j REJECT -p udp";
	    `$command`;
	    $command = "/sbin/ipchains -A output -s $ip --source-port 3201 -j REJECT -p udp";
	    `$command`;

#	    print STDERR "Command = $command\n";

	}

    }

    # Step 3:  Make settings permanent
    if (open(FILE, ">/etc/sysconfig/ipchains")) {

	print FILE "$rules";

	close(FILE);

	# Setup permissions on ipchains file
	`/bin/chown root:root /etc/sysconfig/ipchains`;
	`/bin/chmod 600 /etc/sysconfig/ipchains`;

    }

    if (open(PROFILE, ">/etc/proftpd.conf")) {

	print PROFILE "# Compaq Configurator\n$proftp_file\n";

	close(PROFILE);
    }

    # Reset permissions
    `/bin/chmod o-rxw /etc/proftpd.conf`;

    my $command = "/usr/bin/killall -HUP proftpd";
    my $result = `$command`;    

}

# A sub-routine to configure name resolution
sub updateNameResolution {

    my %system = config::system();

    # Step 1: Re-configure the name servers
    # Build the query
    my $query = 'WHERE
           <DefaultSearchDomain>$dfltsrchdomain</DefaultSearchDomain>
      IN "file:' . $system{'datastore'} . '/network.xml"
      CONSTRUCT $dfltsrchdomain';

    my $dfltsrchdomain = XML::QL->query($query);

    # Build the query
    my $query = 'WHERE
           <NameServer>$nameserver</NameServer>
      IN "file:' . $system{'datastore'} . '/network.xml"
      CONSTRUCT $nameserver~';

    my $result = XML::QL->query($query);

    my @nameservers = split('~', $result);

    my $configFile = "/etc/resolv.conf";

    # Read current entries
    my @lines = ();
    if (open(FILE,"<$configFile")) {
	@lines = <FILE>;
	close(FILE);
    }

    my @newlines = ();

    # Write "resolv.conf" back out
    if (open(FILE, ">$configFile")) {

	my $numOfLines = scalar(@lines);

	for (my $i=0; $i<$numOfLines; $i++) {

	    # Keep commented lines
	    if ($lines[$i] =~ m/^#/) {
		push (@newlines, $lines[$i]);
		next;
	    }
	    # Remove the default search domain
	    elsif ($lines[$i] =~ m/^search/) {
		next;
	    }
	    # Remove existing name servers
	    elsif ($lines[$i] =~ m/^nameserver/) {
		next;
	    }

	}

	# Add the default search domain
	push(@newlines, "search $dfltsrchdomain\n");

	foreach my $nameserver (@nameservers) {
	    push(@newlines, "nameserver $nameserver\n");
	}

	# Write out the file
	foreach my $line (@newlines) {
	    print FILE "$line";
	}

	close(FILE);

    }

    # Step 2: Update static "hosts" entry
    # Build the query
    $query = 'WHERE
           <Host name="$name">$ip</Host>
      IN "file:' . $system{'datastore'} . '/network.xml"
      CONSTRUCT $name<TD>$ip<TR>';

    my $result = XML::QL->query($query);

    my @rows = split('<TR>',$result);

    $configFile = "/etc/hosts";

    # Read current entries
    @lines = ();
    if (open(FILE,"<$configFile")) {
	@lines = <FILE>;
	close(FILE);
    }

    my $date = `/bin/date`;
    if ($lines[0] =~ m/# File Modified by Compaq Configurator/) {
	$lines[0] = "# File Modified by Compaq Configurator - $date\n";
    }
    else {
	push(@lines, "# File Modified by Compaq Configurator - $date\n");
    }

    @newlines = ();

    # Write "hosts" back out
    if (open(FILE, ">$configFile")) {

	my $numOfLines = scalar(@lines);

	for (my $i=0; $i<$numOfLines; $i++) {

	    # Keep commented lines
	    if ($lines[$i] =~ m/^#/) {
		push (@newlines, $lines[$i]);
		next;
	    }
	    # Preserve local hosts
	    elsif ($lines[$i] =~ m/^127/) {
		push (@newlines, $lines[$i]);
		next;
	    }
	    # Remove all other lines
	    else {
		next;
	    }

	}

	# Build the query
	my $query = 'WHERE
          <Network>
           <HostName>$fqdn</HostName>
         </Network>
      IN "file:' . $system{'datastore'} . '/network.xml"
      CONSTRUCT $fqdn';

	my $fqdn = XML::QL->query($query);


	# First grab all IP addresses
	$query = 'WHERE
	    <Interface>
	      <IP>$ip</IP>
	    </Interface>
    	    IN "file:' . $system{'datastore'} . '/network.xml"
      	    CONSTRUCT $ip<TD>';
	
	# Make the query
	$result = XML::QL->query($query);
	
	# Split out the values
	my @ips = split(/<TD>/,$result);

	foreach my $ip (@ips) {
	    $ip =~ s/ //g;
	    push(@newlines, "$ip\t\t$fqdn\n");
	}


	foreach my $row (@rows) {
	    my ($name, $ip) = split('<TD>',$row);

	    # Add the new static hosts
	    push(@newlines, "$ip\t\t$name\n");
	}

	# Write out the file
	foreach my $line (@newlines) {
	    print FILE "$line";
	}

	close(FILE);

    }


}

# A sub-routine to change a network interface
sub updateNetwork {

    my %system = config::system();

    my $configFile = "$system{'sysconfig'}/network";

    # First grab all IP addresses
    my $query = 'WHERE
	    <Interface>
	      <IP>$ip</IP>
	    </Interface>
    	    IN "file:' . $system{'datastore'} . '/network.xml"
      	    CONSTRUCT $ip<TD>';

    # Make the query
    my $result = XML::QL->query($query);
	
    # Split out the values
    my @ips = split(/<TD>/,$result);

    $ips[0] =~ s/ //g;
    $ips[1] =~ s/ //g;

    # Build the query
    my $query = 'WHERE
          <Network>
           <HostName>$fqdn</HostName>
         </Network>
      IN "file:' . $system{'datastore'} . '/network.xml"
      CONSTRUCT $fqdn';

    my $fqdn = XML::QL->query($query);

    # Check to see if we've found the interface
    if ($fqdn eq "") {
	print "No hostname information found!";
	return(0);
    }
    else {

	# Add a notice that the file was modified by the Compaq Configurator
	my $date = `/bin/date`;
	my $message = "# File Modified by Compaq Configurator - $date";

	my @lines = ();

	# Read current configuration
	if (open(FILE, "<$configFile")) {
	    @lines = <FILE>;
	}

	my $numOfLines = scalar(@lines);
	my $hasHostName = 0;

	for (my $i=0; $i<$numOfLines; $i++) {
	    if ($lines[$i] =~ m/^HOSTNAME=/) {
		@lines[$i] = "HOSTNAME=$fqdn\n";
		$hasHostName = 1;
	    }
	}

	# Add a hostname entry if one does not exist
	if (!$hasHostName) {
	    push (@lines, "HOSTNAME=$fqdn\n");
	}

	# Write the file back out
	if (open(FILE,">$configFile")) {
	    foreach my $line (@lines) {
		print FILE "$line";
	    }
	    close(FILE);	    
	}

	# Set hostname
	if (open(FILE,"</etc/hosts")) {
	    
	    @lines = <FILE>;
	    
	    close(FILE);
	    
	    my ($hostname) = split('\.', $fqdn);
	    
	    my $hasLocalAddr = 0;
	    my $hasIP0 = 0;
	    my $hasIP1 = 0;
	    
	    if (open(FILE,">/etc/hosts")) {
		
		# Add a notice that the file was modified by the Compaq Configurator
		my $message = "# File Modified by Compaq Configurator";
		my $date = `/bin/date`;
		
		print FILE "$message - $date
127.0.0.1\tlocalhost.localdomain\tlocalhost
127.0.0.2\t$fqdn\t$hostname
";
		
		foreach my $line (@lines) {
		    
		    if ($line =~ m/File Modified by Compaq Configurator/) {
			next;
		    }
		    elsif ($line =~ m/^127\.0\.0\.1/) {
			next;
		    }
		    elsif ($line =~ m/^127\.0\.0\.2/) {
			next;
		    }
		    elsif ($line eq "\n") {
			next;
		    } 
		    elsif ($line =~ m/^$ips[0]/ && $ips[0] ne "") {
			$line = "$ips[0]\t$fqdn\t$hostname\n";
			$hasIP0 = 1;
		    }
		    elsif ($line =~ m/^$ips[1]/ && $ips[1] ne "") {
			$line = "$ips[1]\t$fqdn\t$hostname\n";
			$hasIP1 = 1;
		    }
		    
		    print FILE "$line";
		}
		
		if (!$hasIP0 && $ips[0] ne "") {
		    print FILE "$ips[0]\t$fqdn\t$hostname\n";
		}
		if (!$hasIP1 && $ips[1] ne "") {
		    print FILE "$ips[1]\t$fqdn\t$hostname\n";
		}
		
		close(FILE);
	    } 

	    my $command = "/bin/hostname $hostname";
	    my $result = `$command`;

	    # Cycle the CIM agents and SNMP to have hostname take effect
	    $result = `$system{'root'}/system/bin/cmafdtn stop`;
	    $result = `$system{'root'}/system/bin/cmafdtn start`;
	    $result = `$system{'root'}/system/bin/snmpd stop`;
	    $result = `$system{'root'}/system/bin/snmpd start`;
	    
	}

    }

    return(1);

}

# A sub-routine to change a network interface
sub updateInterface {

    $| = 1;

    my $interface = shift || return(0);

    my %system = config::system();

    my $gateway = "";

    # Choose corresponding config file for interface
    # We do it this way because suid perl requires it
    my $if = "";
    if ($interface eq "eth0") {
       $if = "eth0";
    }
    elsif ($interface eq "eth1") {
       $if = "eth1";
    }
    # A bad interface name
    else {
	exit(0);
    }

    my $configFile = "$system{'netscripts'}/ifcfg-$if";

    # Build the query
    my $query = 'WHERE
         <Interface if="' . $if . '">
           <Enabled>$enabled</Enabled>
           <Mode>$mode</Mode>
           <IP>$ip</IP>
           <NetMask>$netmask</NetMask>
           <Network>$network</Network>
           <Broadcast>$bcast</Broadcast>
         </Interface>
      IN "file:' . $system{'datastore'} . '/network.xml"
      CONSTRUCT $enabled<TD>$ip<TD>$mode<TD>$netmask<TD>$network<TD>$bcast';

    my $result = XML::QL->query($query);

    # Check to see if we've found the interface
    if ($result eq "") {
	print "No interface configuration information found for $if!";
	return(0);
    }
    else {

	my @values = split(/<TD>/, $result);

	my $enabled = $values[0] || "No";
	my $ip = $values[1] || "";
	my $mode = $values[2] || "";
	my $netmask = $values[3] || "";
	my $network = $values[4] || "";
	my $bcast = $values[5] || "";

	print 
	
	# Build the query
	my $query = 'WHERE
          <DefaultGateway>$gw</DefaultGateway>
      IN "file:' . $system{'datastore'} . '/network.xml"
      CONSTRUCT $gw';

	$gateway = XML::QL->query($query);

	print STDERR "Extracted GW of $gateway\n";

	# Add a notice that the file was modified by the Compaq Configurator
	my $message = "# File Modified by Compaq Configurator";
	my $date = `/bin/date`;

	my @lines = ();
	$lines[0] = "$message - $date";
	$lines[1] = "DEVICE=$if";

	if ($enabled eq "Yes") {
	    push(@lines, "ONBOOT=yes");
	}
	else {
	    push(@lines, "ONBOOT=no");
	}

	if ($mode eq "DHCP") {
	    push(@lines, "BOOTPROTO=dhcp");
	}
	else {
	    push(@lines, "IPADDR=$ip");
	    push(@lines, "NETMASK=$netmask");
	    push(@lines, "NETWORK=$network");
	    push(@lines, "BROADCAST=$bcast");
	    push(@lines, "GATEWAY=$gateway");
	    print STDERR "Pushed gateway onto new lines $gateway (Enabled=$enabled)\n";
	}

#	push(@lines, "USERCTL=yes");

	# Write the file back out
	if (open(FILE,">$configFile")) {

	    foreach my $line (@lines) {

		print FILE "$line\n";
#		print STDERR "  $line\n";
	    }
	
	    close(FILE);
	
	}

	# Stop that interface
	my $command = "/sbin/ifconfig $if down";
	`$command`;

	# Restart only if enabled
	if ($enabled eq "Yes") {

	    my $command = "";

	    if ($mode eq "DHCP" || $mode eq "BOOTP") {
		my $ethpid = `/bin/cat /var/run/dhcpd-$if.pid`;
		$command = "/bin/kill -9 $ethpid";
		`$command`;
		`/bin/rm /var/run/dhcpd-$if.pid`;
		$command = "/sbin/dhcpcd -B -r -t 30 -i TaskSmartW2200 $if";
		`$command`;

		# Grab cached information in case dhcp server didn't respond
		my $ip = `/bin/cat /etc/dhcpc/dhcpcd-$if.info | /bin/grep IPADDR`;
		my $netmask = `/bin/cat /etc/dhcpc/dhcpcd-$if.info | /bin/grep NETMASK`;

		$ip =~ s/\n//g;
		$ip =~ s/IPADDR=//g;
		$netmask =~ s/\n//g;
		$netmask =~ s/NETMASK=//g;
		
		`/sbin/ifconfig $if up netmask $netmask $ip`;
	    }
	    else {
		# We execute this twice because of a little bug in ifconfig
		$command = "/sbin/ifconfig $if up netmask $netmask $ip";
		`$command`;
		`$command`;
	    }

	    # Reset the default gateway
  	    if ($gateway ne "") {

		print STDERR "Manually adding default gw of $gateway\n";
  		$command = "/sbin/route del default";
  		$result = `$command`;
  		$command = "/sbin/route -n add default gw $gateway";
  		$result = `$command`;
  	    }
		
	    # Do post configuration updates to the XML
	    my $ifconfig = `/sbin/ifconfig $if`;
	    my $route = `/sbin/route -n | /bin/grep ^0.0.0.0`;
	    my @values = split(/\s*/, $route);
	    $gateway = $values[1];

	    $ifconfig =~ /inet addr:([\d\.]*)/;
	    my $ip = $1;
	    $ifconfig =~ /Mask:([\d\.]*)/;
	    my $netmask = $1;
	    $ifconfig =~ /Bcast:([\d\.]*)/;
	    my $bcast = $1;
	    $ifconfig =~ m/HWaddr ([\dABCDEF]{2}:[\dABCDEF]{2}:[\dABCDEF]{2}:[\dABCDEF]{2}:[\dABCDEF]{2}:[\dABCDEF]{2})/;
	    my $macaddr = $1;

	    # Update the XML
	    my $parser = new XML::DOM::Parser;
	    my $doc = $parser->parsefile("$system{'datastore'}/network.xml");
	    
	    my $interfaces = $doc->getElementsByTagName("Interface");
	    my $numOfInterfaces = $interfaces->getLength;
	    
	    my $interfaceNode = "";
	    
	    for (my $i=0; $i<$numOfInterfaces; $i++) {
		
		my $thisIF = eval { $interfaces->item($i)->getAttribute("if"); } || "";
		
		# Did we find the right node?
		if ($thisIF eq $if) {
		    $interfaceNode = $interfaces->item($i);
		}
		
	    }
	    
	    if ($interfaceNode eq "") {
		print "<H1><FONT color=red>Unknown interface: $interface</FONT></H1>";
	    }
	    else {
		
	      datastore::textNodeUpdate($interfaceNode->getElementsByTagName("MACAddress")->item(0), "$macaddr");
	      datastore::textNodeUpdate($interfaceNode->getElementsByTagName("IP")->item(0), "$ip");
	      datastore::textNodeUpdate($interfaceNode->getElementsByTagName("NetMask")->item(0), "$netmask");
	      datastore::textNodeUpdate($interfaceNode->getElementsByTagName("Broadcast")->item(0), "$bcast");
	      datastore::textNodeUpdate($interfaceNode->getElementsByTagName("DefaultGateway")->item(0), "$gateway");
		
		# Write out the file
		$doc->printToFile("$system{'datastore'}/network.xml");
		
		$doc->dispose();
	    }	    
	}
	# Really disable the interface
	else {
	    my $command = "/sbin/ifconfig $if 0.0.0.0";
	    `$command`;
	}
    }

    # Update the virutal sites to reflect the new IP address
    updateServices();
    updateVirtualSiteIPs();
    restartApache();

    return(1);

}

# A sub-routine to setup network services
sub updateNetworkServices {

    $| = 1;

    my %system = config::system();

    # Build the query
    my $query = 'WHERE
         <Interface if="$if">
           <Enabled>$enabled</Enabled>
           <Mode>$mode</Mode>
           <IP>$ip</IP>
           <NetMask>$netmask</NetMask>
           <Network>$network</Network>
           <Broadcast>$bcast</Broadcast>
           <Telnet>$telnet</Telnet>
           <FTP>$ftp</FTP>
           <POP3>$pop3</POP3>
           <SSH>$ssh</SSH>
           <IMAP>$imap</IMAP>
           <NFS>$nfs</NFS>
         </Interface>
      IN "file:' . $system{'datastore'} . '/network.xml"
      CONSTRUCT $telnet<TD>$pop3<TD>$ssh<TD>$imap<TD>$nfs<TR>';

    my $result = XML::QL->query($query);

    # Check to see if we've found the interface
    if ($result eq "") {
	print "No network services configuration information found!";
	return(0);
    }
    else {

	my @nics = split(/<TR>/, $result);

	foreach my $nic (@nics) {
	}

    }

    return(1);

}

# A sub-routine to add an NFS mount point to the system
sub addNFSMount {

    my $connectString = shift || return;
    my $mountPoint = shift || return;

    print STDERR "NFS add - $connectString\t$mountPoint\n";
    
    my %system = config::system();

    # Safe-gaurd against shell manipulation
    $connectString =~ s/;//;
    $mountPoint =~ s/;//;
    
    # Step 1 - Check to see if the directory exists and
    # make the directory if it does not exist.
    if (!(-d "$mountPoint")) {
	
	my $command = "/bin/mkdir $mountPoint";
	my $result = `$command`;
	
	if (!(-d "$mountPoint")) {
	    return("MOUNTPOINT_COULDNOTCREATE");
	}

    }
    
    # Step 2 - Add the mount to /etc/fstab
    my @lines = ();
    if (open(FILE,"</etc/fstab")) {
	
	@lines = <FILE>;
	
	close(FILE);

	# Add the new NFS mount
	if (open(FILE, ">/etc/fstab")) {
	    
	    foreach my $line (@lines) {

		# Don't duplicate the NFS mount
		if ($line =~ m/$connectString \t $mountPoint/) {
		    next;
		}
		else {
		    print FILE $line;
		}
	    }
	}

	print FILE "$connectString \t $mountPoint \t nfs \t user,rsize=8192,wsize=8192,timeo=14,intr\n";	
    }

    # Step 3 - Attempt to manually mount the NFS mount point
    my $command = "/bin/mount $mountPoint";
    my $result = `$command`;

    print STDERR "$result\n";

    # Step 4 - Fix-up for tighter security
    my @lines = ();
    if (open(FILE,"</etc/fstab")) {
	
	@lines = <FILE>;
	
	close(FILE);

	# Search for the previosuly added line and fix it
	if (open(FILE, ">/etc/fstab")) {
	    
	    foreach my $line (@lines) {

		if ($line eq "$connectString \t $mountPoint \t nfs \t user,rsize=8192,wsize=8192,timeo=14,intr\n") {
		    print FILE "$connectString \t $mountPoint \t nfs \t rsize=8192,wsize=8192,timeo=14,intr\n";
		}
		else {
		    print FILE $line;
		}
	    }
	}
    }
    
    return("");
    
}

# A sub-routine to delete an NFS mount point
sub delNFSMount {

    my %system = config::system();

    my $connectString = shift || return;
    my $mountPoint = shift || return;

    # Safe-gaurd against shell manipulation
    $connectString =~ s/;//;
    $mountPoint =~ s/;//;

    # Step 1 - Attempt to manually un-mount the NFS mount point
    my $command = "$system{'root'}/system/bin/umount $mountPoint";
    my $result = `$command`;

    # Step 2 - Fix-up the fstab to allow user dis-mount
    my @lines = ();
    if (open(FILE,"</etc/fstab")) {

	@lines = <FILE>;

	close(FILE);
    }

    if (open(FILE, ">/etc/fstab")) {

	foreach my $line (@lines) {

	    # Skip the line we are tring to remove
	    if ($line =~ m/^$connectString(\W+)$mountPoint(\W+)/) {
		next;
	    }
	    else {
		print FILE $line;
	    }

	}
    }

    $command = "/bin/rmdir $mountPoint";
    $result = `$command`;

    return("");

}

# A sub-routine to update the NFS mounts as described in the datastore
sub updateNFSMounts {

    my %system = config::system();

    my $parser = new XML::DOM::Parser;
    my $doc = $parser->parsefile("$system{'datastore'}/network.xml");

    foreach my $mount ($doc->getElementsByTagName("Mount")) {

	my $remote = eval { $mount->getElementsByTagName("Remote")->item(0)->getFirstChild->getData(); } || "";
	my $local = eval { $mount->getElementsByTagName("Local")->item(0)->getFirstChild->getData(); } || "";

	addNFSMount($remote, $local);

    }

    $doc->dispose();

    return(1);
}

# A sub-routine to add a user to the system
sub addUser {

    my $loginname = shift || return(0);
    my $passwd = shift || return(0);

    my $userID = "";
    my $ssh = "";
    my $ftp = "";
    my $telnet = "";
    my $pop3 = "";
    my $imap = "";
    my $diskquota = "";

    my %system = config::system();

    # Build the query
    my $query = 'WHERE
           <LoginName>' . $loginname . '</LoginName>
           <Status>$status</Status>
      IN "file:' . $system{'datastore'} . '/users.xml"
      CONSTRUCT $status';

    my $result = XML::QL->query($query);

    # Check to see if we've found the interface
    if ($result eq "") {
	print "This user is not valid ($loginname)!";
	return();
    }
    else {

	# Add the User
	my $command = "/usr/sbin/adduser $loginname -p $passwd";
	my $result = `$command`;

	# Reset permissions on password files
	$command = "$system{'root'}/system/bin/chgrp cpqcfg /etc/passwd* /etc/shadow*";
	$result = `$command`;

	print STDERR "Results from password permission change = $result\n";

	# Do post configuration updates to the XML
	$userID = `/usr/bin/id -u $loginname`;
	$userID =~ s/\n//g;
	my $groupID = `/usr/bin/id -g $loginname`;
	$groupID =~ s/\n//g;

	# Build public_html directory and make apache able to see it
	$command = "/bin/mkdir /home/$loginname/public_html";
	$result = `$command`;
	$command = "/bin/chmod +x /home/$loginname";
	$result = `$command`;

	# Build temporary place holder for public_html
	if (open(INDEXFILE, ">/home/$loginname/public_html/index.html")) {

	    print INDEXFILE "<HTML>
<HEAD>
  <TITLE>Future site for \"$loginname\"</TITLE>
</HEAD>

<BODY bgcolor=white>
  <CENTER><H2>Future site for \"$loginname\"</H2></CENTER>
</BODY>

</HTML>
";

	    close(INDEXFILE);

	}

	# Fix-up permissions
	$command = "/bin/chown $loginname:$loginname /home/$loginname";
	$result = `$command`;
	$command = "/bin/chown -R $loginname:$loginname /home/$loginname";
	$result = `$command`;

	# Update the XML
	my $parser = new XML::DOM::Parser;
	my $doc = $parser->parsefile("$system{'datastore'}/users.xml");

	my $users = $doc->getElementsByTagName("User");
	my $numOfUsers = $users->getLength;

	my $userNode = "";

	# Search through the users
	for (my $i=0; $i<$numOfUsers; $i++) {

	    my $thisUser = eval { $users->item($i)->getElementsByTagName("LoginName")->item(0)->getFirstChild->getData(); } || "";

	    # Did we find the right node?
	    if ($thisUser eq $loginname) {
		$userNode = $users->item($i);
	    }

	}
	
	if ($userNode eq "") {
	    print "<H1><FONT color=red>Unknown user: $loginname</FONT></H1>";
	}
	else {

	    # Populate variables
	    $ssh = eval { $userNode->getElementsByTagName("Services")->item(0)->getElementsByTagName("SSH")->item(0)->getFirstChild->getData(); } || "";
	    $ftp = eval { $userNode->getElementsByTagName("Services")->item(0)->getElementsByTagName("FTP")->item(0)->getFirstChild->getData(); } || "";
	    $telnet = eval { $userNode->getElementsByTagName("Services")->item(0)->getElementsByTagName("Telnet")->item(0)->getFirstChild->getData(); } || "";
	    $pop3 = eval { $userNode->getElementsByTagName("Services")->item(0)->getElementsByTagName("POP3")->item(0)->getFirstChild->getData(); } || "";
	    $imap = eval { $userNode->getElementsByTagName("Services")->item(0)->getElementsByTagName("IMAP")->item(0)->getFirstChild->getData(); } || "";
	    $diskquota = eval { $userNode->getElementsByTagName("DiskQuota")->item(0)->getFirstChild->getData(); } || "";

 	    datastore::attributeNodeUpdate($userNode->getAttributeNode("uid"), $userID);
 	    datastore::attributeNodeUpdate($userNode->getAttributeNode("gid"), $groupID);
	    
	    # Write out the file
	    $doc->printToFile("$system{'datastore'}/users.xml");
	    
	}


	# Update the disk quota for this user
	my $dev = Quota::getqcarg("/home");
	my $diskquotablocks = $diskquota * 1.15;
	print STDERR "Add user quota of $diskquotablocks";
      Quota::setqlim($dev,$userID,$diskquotablocks,$diskquotablocks,0,0,0);

	$doc->dispose();

	# Update the group.xml file
	my $parser = new XML::DOM::Parser;
	my $doc = $parser->parsefile("$system{'datastore'}/groups.xml");

      datastore::addGroup($doc, $groupID, $loginname, $loginname, "", $userID);

	$doc->printToFile("$system{'datastore'}/groups.xml");

	$doc->dispose();

	# check the services and enable them on the user
	# SSH
	if (open(FILE,"</etc/sshusers")) {

	    my @sshlines = <FILE>;

	    close (FILE);

	    open (FILE, ">/etc/sshusers");

	    # Run through each line if the file and skip this user
	    foreach my $sshline (@sshlines) {
		
		$sshline =~ s/\n//g;

		if ($loginname eq $sshline) {
		    next;
		}

		print FILE "$sshline\n";

	    }

	    # Add them if they're not supposed to have access
	    if ($ssh eq "No") {
		print FILE "$loginname\n";
	    }

	    close(FILE);
	}

	# FTP
	if (open(FILE, "</etc/ftpusers")) {
	    my @ftplines = <FILE>;
	    close(FILE);

	    open(FILE, ">/etc/ftpusers");

	    foreach my $ftpline (@ftplines) {

		$ftpline =~ s/\n//g;
		if ($loginname eq $ftpline) {
		    next;
		}
		print FILE "$ftpline\n";

	    }
	    # Add them if they'r not supposed to 
	    if ($ftp eq "No") {
		print FILE "$loginname\n";
	    }
	    close(FILE);
	}
	

	# Telnet
	if (open(FILE, "</etc/telnetusers")) {

	    my @telnetlines = <FILE>;
	    close(FILE);
	    
	    open(FILE, ">/etc/telnetusers");

	    foreach my $telnetline (@telnetlines) {

		$telnetline =~ s/\n//g;

		if ($loginname eq $telnetline) {
		    next;
		}

		print FILE "$telnetline\n";

	    }

	    # Add them if they're not supposed to
	    if ($telnet eq "No") {
		print FILE "$loginname\n";
	    }
	    close(FILE);

	}

	# POP3
	if (open(FILE, "</etc/pop3users")) {

	    my @pop3lines = <FILE>;
	    close(FILE);

	    open(FILE, ">/etc/pop3users");
	    
	    foreach my $pop3line (@pop3lines) {
		$pop3line =~ s/\n//g;
		if ($loginname eq $pop3line) {
		    next;
		}
		print FILE "$pop3line\n";
	    }
	    # Add them if they're not supposed to
	    if ($pop3 eq "No") {
		print STDERR "  POP3 user being denied access $loginname\n";
		print FILE "$loginname\n";
	    }
	    close(FILE);
	}  
	# IMAP
	if (open(FILE, "</etc/imapusers")) {

	    my @imaplines = <FILE>;
	    close(FILE);

	    open(FILE, ">/etc/imapusers");
	    
	    foreach my $imapline (@imaplines) {
		$imapline =~ s/\n//g;
		if ($loginname eq $imapline) {
		    next;
		}
		print FILE "$imapline\n";
	    }
	    # Add them if they're not supposed to
	    if ($imap eq "No") {
		print STDERR "  IMAP user being denied access $loginname\n";
		print FILE "$loginname\n";
	    }
	    close(FILE);
	}

    }

    return($userID);

}

# A sub-routine to update user attributes
sub updateUser {

    print STDERR "Beginning user update\n";

    my $loginname = shift || return(0);
    my $password = shift || "";

    my %userinfo = datastore::userInfo($loginname);


#    print STDERR "Got password of $password\n";

    if ($password ne "") {
	my $command = "/usr/sbin/usermod -p $password $loginname";
	`$command`;
    }

    my %system = config::system();

    # Parse the user file
    my $parser = new XML::DOM::Parser;
    my $doc = $parser->parsefile("$system{'datastore'}/users.xml");

    # Locate this user
    my $users = $doc->getElementsByTagName("User");
    my $userNode = "";
    my $numOfUsers = $users->getLength();
    
    for (my $i=0; $i < $numOfUsers; $i++) {
	
	my $thisLoginName = $users->item($i)->getElementsByTagName("LoginName")->item(0)->getFirstChild->getData();
	
	# Did we find it?
	if ($thisLoginName eq $loginname) {
	    $userNode = $users->item($i);
	    last;
	}
	
    }

    my $status = "";
    my $diskquota = "";
    my $ssh = "";
    my $ftp = "";
    my $telnet = "";
    my $pop3 = "";
    my $imap = "";

    # If we found the user, then populate the variables for edit
    if ($userNode ne "") {
    
	# Populate variables
	$status = eval { $userNode->getElementsByTagName("Status")->item(0)->getFirstChild->getData(); } || "";
	$diskquota = eval { $userNode->getElementsByTagName("DiskQuota")->item(0)->getFirstChild->getData(); } || "";
	$ssh = eval { $userNode->getElementsByTagName("Services")->item(0)->getElementsByTagName("SSH")->item(0)->getFirstChild->getData(); } || "";
	$ftp = eval { $userNode->getElementsByTagName("Services")->item(0)->getElementsByTagName("FTP")->item(0)->getFirstChild->getData(); } || "";
	$telnet = eval { $userNode->getElementsByTagName("Services")->item(0)->getElementsByTagName("Telnet")->item(0)->getFirstChild->getData(); } || "";
	$pop3 = eval { $userNode->getElementsByTagName("Services")->item(0)->getElementsByTagName("POP3")->item(0)->getFirstChild->getData(); } || "";
	$imap = eval { $userNode->getElementsByTagName("Services")->item(0)->getElementsByTagName("IMAP")->item(0)->getFirstChild->getData(); } || "";

    }
    else {
	print STDERR "Could not find the user node!!!\n";
	return(0);
    }

    my $password = "";

    print STDERR "About to open shadow file\n";

    # Grab the password for this user
    if (open(FILE, "</etc/shadow")) {
	
	my $foundUser = 0;
	my @lines = <FILE>;

	print STDERR "Shadow open and read\n";
	
	foreach my $line (@lines) {
	    
	    if ($line =~ m/^$loginname:([^:]*):/) {
		$password = $1;
	    }
	}
    }

    print STDERR "Finished with shadow file\n";

    # Check to see if we've found the right user
    if ($password eq "") {
	print "This user is not valid ($loginname)!";
	return();
    }
    else {

	print STDERR "Updating user $loginname\n";

	# Update password
	my $command = "/usr/sbin/usermod -p $password $loginname";
	my $result = `$command`;

	print STDERR "Status = $status\n";

	# Activate/In-activate user
	if ($status eq "Active") {

	    $command = "/usr/sbin/usermod -U $loginname";

	}
	else {

	    # First make sure they are completely off the system
	    $command = "/bin/ps -U $userinfo{'uid'} -o pid | /bin/grep \\\\.*";
	    $result = `$command`;
	    
	    print STDERR "PIDS for user $userinfo{'uid'} $command\n $result\n";
	    
	    # Gather PIDs and drop the header "PID"
	    my @pids = split("\n", $result);
	    shift(@pids);
	    
	    # Kill all user processes
	    foreach my $pid (@pids) {
		$command = "/bin/kill -9 $pid";
		$result = `$command`;
#		print STDERR "$command\n $result\n";
	    }

	    $command = "/usr/sbin/usermod -L $loginname";

	    delHTPasswd ($loginname, "$system{'root'}/siteAdmin/.htpasswd");	
	    delHTPasswd ($loginname, "$system{'root'}/user/.htpasswd");

	}

	$result = `$command`;

	print STDERR "HTPasswd updated\n";

	# Update disk quotas
	print STDERR "About to update quotas\n";
	my $dev = Quota::getqcarg("/home");
	my $diskquotablocks = $diskquota * 1.15;
	print STDERR "Changed quota of $diskquota kilobytes into $diskquotablocks blocks.\n";
      Quota::setqlim($dev,$userinfo{'uid'},$diskquotablocks,$diskquotablocks,0,0,0);
	print STDERR "Quotas updated...$!\n";
	

    }

    # check the services and enable them on the user
    # SSH
    if (open(FILE,"</etc/sshusers")) {

	my @sshlines = <FILE>;

	close (FILE);

	open (FILE, ">/etc/sshusers");

	# Run through each line if the file and skip this user
	foreach my $sshline (@sshlines) {
	    
	    $sshline =~ s/\n//g;

	    if ($loginname eq $sshline) {
		next;
	    }

	    print FILE "$sshline\n";

	}

	# Add them if they're not supposed to have access
	if ($ssh ne "Yes") {
	    print FILE "$loginname\n";
	}

	close(FILE);
    }

    # FTP
    if (open(FILE, "</etc/ftpusers")) {
	my @ftplines = <FILE>;
	close(FILE);

	open(FILE, ">/etc/ftpusers");

	foreach my $ftpline (@ftplines) {

	    $ftpline =~ s/\n//g;
	    if ($loginname eq $ftpline) {
		next;
	}
	    print FILE "$ftpline\n";

	}
	# Add them if they're not supposed to 
	if ($ftp ne "Yes") {
	    print FILE "$loginname\n";
	}
	close(FILE);
    }
   

    # Telnet
    if (open(FILE, "</etc/telnetusers")) {

	my @telnetlines = <FILE>;
	close(FILE);
	
	open(FILE, ">/etc/telnetusers");

	foreach my $telnetline (@telnetlines) {

	    $telnetline =~ s/\n//g;

	    if ($loginname eq $telnetline) {
		next;
	    }

	    print FILE "$telnetline\n";

	}

	# Add them if they're not supposed to
	if ($telnet ne "Yes") {
	    print FILE "$loginname\n";
	}
	close(FILE);

    }

    # POP3
    if (open(FILE, "</etc/pop3users")) {
	my @pop3lines = <FILE>;
	close(FILE);

	open(FILE, ">/etc/pop3users");
    
	foreach my $pop3line (@pop3lines) {
	    $pop3line =~ s/\n//g;
	    if ($loginname eq $pop3line) {
		next;
	    }
	    print FILE "$pop3line\n";
	}
	# Add them if they're not supposed to
	if ($pop3 ne "Yes") {
	    print FILE "$loginname\n";
	}
	close(FILE);
    }  

    # IMAP

    if (open(FILE, "</etc/imapusers")) {
	my @imaplines = <FILE>;
	close(FILE);

	open(FILE, ">/etc/imapusers");
    
	foreach my $imapline (@imaplines) {
	    $imapline =~ s/\n//g;
	    if ($loginname eq $imapline) {
		next;
	    }
	    print FILE "$imapline\n";
	}
	# Add them if they're not supposed to
	if ($imap ne "Yes") {
	    print FILE "$loginname\n";
	}
	close(FILE);
    }

    # Reset permissions on password files
    my $command = "$system{'root'}/system/bin/chgrp cpqcfg /etc/passwd* /etc/shadow*";
    my $result = `$command`;

    print STDERR "Results from passwd fix = $result\n";

    updateSendmailGenericsTable();
    
}

# A sub-routine to remove a user from the system
sub delUser {

    my $loginname = shift || "";

    my %system = config::system();

    my %userInfo = datastore::userInfo($loginname);

    # First make sure they are completely off the system
    my $command = "/bin/ps -U $userInfo{'uid'} -o pid | /bin/grep \\\\.*";
    my $result = `$command`;

    print STDERR "PIDS for user $userInfo{'uid'} $command\n $result\n";

    # Gather PIDs and drop the header "PID"
    my @pids = split("\n", $result);
    shift(@pids);

    # Kill all user processes
    foreach my $pid (@pids) {
	$command = "/bin/kill -9 $pid";
	$result = `$command`;
#	print STDERR "$command\n $result\n";
    }

    # Remove the user from the system
    $command = "/usr/sbin/userdel -r $loginname";
    $result = `$command`;
    $command = "/usr/sbin/groupdel $loginname";
    $result = `$command`;

    print STDERR "User $loginname Deleted from the system!\n";

    # Remove the user from the XML
    datastore::delUser($loginname);

    updateSendmailGenericsTable();

    print STDERR "User completely deleted!\n";

    return(1);
}

# A sub-routine to add a group to the system
sub addGroup {

    my $shortname = shift || return("");
    my $name = shift || "";
    my $diskquota = shift || "0";
    my @members = @_;

    my %system = config::system();

    print STDERR "MEMBERS = $members[0]\n";

    my $groupID = "";

    # Parse the group file
    my $parser = new XML::DOM::Parser;
    my $doc = $parser->parsefile("$system{'datastore'}/groups.xml");

    # Search for this group
    my $foundNode = "";
    foreach my $groupNode ($doc->getElementsByTagName("Group")) {

	my $thisName = eval { $groupNode->getElementsByTagName("ShortName")->getFirstChild->getData(); } || "";

	if ($thisName eq $shortname) {
	    $foundNode = $groupNode;
	}
    }

    # If we didn't find the group, then add it
    if ($foundNode eq "") {

	# Add the Group
	my $command = "/usr/sbin/groupadd $shortname";
	my $result = `$command`;

	# Update the shadow file
	$result = `/usr/bin/grpconv`;

	# Grab the group ID
	$result = `/bin/cat /etc/group | /bin/grep ^$shortname:`;
	my @row = split(/:/, $result);
	$groupID = $row[2];

	# Update the XML
	my $parser = new XML::DOM::Parser;
	my $doc = $parser->parsefile("$system{'datastore'}/groups.xml");

	print STDERR "\n$groupID $shortname $name Members = @members\n";

        datastore::addGroup($doc, $groupID, $shortname, $name, $diskquota, @members);

	datastore::XMLtoFile($doc,"$system{'datastore'}/groups.xml");

	$doc->dispose();

	updateGroup($shortname);

    }

    return($groupID);

}

# A sub-routine to add a group to the system
sub addVSGroup {

    my $shortname = shift || return("");
    my $name = shift || "";
    my $diskquota = shift || "0";
    my @members = @_;

    print STDERR "Adding Virtual Site Group\nMEMBERS = @members\n";

    my $groupID = "";

    my %system = config::system();

    # Build a temporay group name
    srand();
    my $tempGroupName = "tempgroup" . int(rand(5000));

    # Add the Temporary Group
    my $command = "/usr/sbin/groupadd $tempGroupName";
    my $result = `$command`;
    
    # Grab the group ID
    $result = `/bin/cat /etc/group | /bin/grep ^$tempGroupName:`;
    my @row = split(/:/, $result);
    $groupID = $row[2];

    $shortname = "virtualsite_$groupID";
    
    # Change the Temporary Group to the Real Group
    my @lines = ();
    if (open(FILE, "/etc/group")) {
	
	@lines = <FILE>;
	
	# Replace the group name
	foreach my $line (@lines) {
	    if ($line =~ m/$tempGroupName:/) {
		$line =~ s/$tempGroupName:/$shortname:/;
		last;
	    }
	}
	
	close(FILE);
	
	if (open(FILE, ">/etc/group")) {
	    
	    # Write the file back out
	    foreach my $line (@lines) {
		print FILE "$line";
	    }
	    
	    close(FILE);
	    
	}
	
	# Update the shadow file
	$result = `/usr/bin/grpconv`;
	
	# Update the XML
	my $parser = new XML::DOM::Parser;
	my $doc = $parser->parsefile("$system{'datastore'}/groups.xml");
	
	print STDERR "\n$groupID $shortname $name Members = @members\n";
	
      datastore::addGroup($doc, $groupID, $shortname, $name, $diskquota, @members);
	
      datastore::XMLtoFile($doc,"$system{'datastore'}/groups.xml");
	
	$doc->dispose();

	updateGroup($shortname);

    }
    
    return($groupID);
    
}


# A sub-routine to update the members of a group on the system
sub updateGroup {

    my %system = config::system();

    my $groupname = shift || return;

    print STDERR "Updating group $groupname\n";

    # Grab members of the group
    my $parser = new XML::DOM::Parser;
    my $doc = $parser->parsefile("$system{'datastore'}/groups.xml");

    my $groupNode = "";

    # Locate the group node for this group
    foreach my $group ($doc->getElementsByTagName("Group")) {

	my $name = eval { $group->getElementsByTagName("ShortName")->item(0)->getFirstChild->getData(); } || "";

	if ($name eq $groupname) {
	    $groupNode = $group;
	    last;
	}

    }

    # Check to make sure we found a node
    if ($groupNode eq "") {
	return;
    }

    # Open and read the group file
    my $configFile = "/etc/group";
    my @lines = ();

    # Read in the current group file
    if (open(FILE, "<$configFile")) {

	@lines = <FILE>;

	close(FILE);
    
	# Search for this group
	foreach my $line (@lines) {

	    # skip comments
	    if ($line =~ m/^#/) {
		next;
	    }
	    # else if this groups line is found
	    elsif ($line =~ m/^$groupname:/) {

		print STDERR "  *** Found the correct group to update! ($line)\n";

		my @values = split(':',$line);
		my @users = split(',', $values[3]);

#		print STDERR "Current members of this group $values[3]\n";

		# Remove all users from the group first
		foreach my $user (@users) {
		    print STDERR "    --- Removing $user\n";
		    delUserFromOSGroup($user, $groupname);
		}

		# Add the correct users back
		my $members = eval { $groupNode->getElementsByTagName("UserID"); } || "";
		my $numOfMembers = $members->getLength;

#		print STDERR "  Number of Members = $numOfMembers\n";
		for (my $j=0; $j < $numOfMembers; $j++) {

		    my $memberID = eval { $members->item($j)->getFirstChild->getData(); } || "";
		    my %userInfo = datastore::userInfoByID($memberID);

		    print STDERR "    ---- Adding $userInfo{'loginname'} to $groupname\n";

		    addUserToOSGroup($userInfo{'loginname'}, $groupname);
		}

		last;

	    }

	}

    }

    # Step 2 - Update group quota
    if ($groupNode ne "") {

	my $gid = $groupNode->getAttribute("id");

	my $diskquota = eval { $groupNode->getElementsByTagName("DiskQuota")->item(0)->getFirstChild->getData; } || "";

	my $dev = Quota::getqcarg("/home");
	my $diskquotablocks = $diskquota * 1.15;
#	print STDERR "Changed quota of $diskquota kilobytes into $diskquotablocks blocks.\n";
      Quota::setqlim($dev,$gid,$diskquotablocks,$diskquotablocks,0,0,0,1);

#	print STDERR "Updating group $gid quota $diskquota\n";

    }

    # Update shadow group
    `/usr/sbin/grpconv`;
    
}

# A sub-routine to remove a group from the system
sub delGroup {

    my $groupname = shift || return;

    my %system = config::system();

#    print STDERR "Removing group from the system\n";
    # Remove the user from the system
    my $command = "/usr/sbin/groupdel $groupname";
    my $result = `$command`;
#    print STDERR "Group removed from the system\n";

    # Double check that the group really was deleted
#    print STDERR "Double check\n";
    my $groupcheck = `/bin/cat /etc/group | /bin/grep ^$groupname:`;
    my $usercheck = `/bin/cat /etc/passwd | /bin/grep ^$groupname:`;
#    print STDERR "END OF Double check\n";

    # Check to make sure that the user wasn't trying to delete
    # a primary user group.
    if ($groupcheck eq "") {

	print STDERR "Group $groupname Deleted from the system!\n";

	# Update the XML
	my $parser = new XML::DOM::Parser;
	my $doc = $parser->parsefile("$system{'datastore'}/groups.xml");
	
	# Remove the group from the XML
	print STDERR "Datastore group delete\n";
      datastore::delGroup($doc, $groupname);
	print STDERR "END OF Datastore group delete\n";

	# Write out the file
	$doc->printToFile("$system{'datastore'}/groups.xml");
	
	$doc->dispose();
	
	print STDERR "Group completely deleted!\n";
    }
    elsif ($usercheck ne "") {
	print error::throwError("GROUP_PRIMARY_USERS","ANY",$groupname);
    }

    return(1);
}

# A sub-routine to shut the system down
sub shutdown {

    my $postAction = shift || "";
    my $when = shift || "";

    my $time = "";
    if ($when = "now") {
	$time = "now";
    }

    my $command = "";

    # Determine the type of shutdown
    if ($postAction eq "halt") {
	$command = "/sbin/shutdown -h $time";
    }
    else {
	$command = "/sbin/shutdown -r $time";
    }

    # Do the shutdown
    `$command`;

    return (1);

}

# A sub-routine that adds a virtual site to the system
sub addVirtualSite {

    my %system = config::system;

    my $sitename = shift || return(0);
    my $diskquota = shift || "0";
    my @members = @_;
    
    # First grab all IP addresses
    my $query = 'WHERE
	    <Interface>
	      <IP>$ip</IP>
	    </Interface>
    	    IN "file:' . $system{'datastore'} . '/network.xml"
      	    CONSTRUCT $ip<TD>';

    # Make the query
    my $result = XML::QL->query($query);
    
    # Split out the values
    my @ips = split(/<TD>/,$result);
    my $iplist = join(' ', @ips);

    # If the vhosts file does not exists, populate it with the root web
    if  ((-s $system{'vhostsfile'}) <= 0) {

	`/bin/rm /usr/local/frontpage/we80.cnf`;

	open(VHOSTFILE, ">$system{'vhostsfile'}");

	print VHOSTFILE "<VirtualHost $iplist>
   DocumentRoot \"/var/www/html\"\n";

	my $fpextensions = `/bin/cat $system{'datastore'}/apache.xml | /bin/grep \\\<FrontPageExtensions\\\>Yes`;

	# If front page is on, add it to the root web
	if ($fpextensions ne "") {

	    print VHOSTFILE "

  <Location \"/_vti_pvt\">
    Order deny,allow
    Deny from all
  </Location>
  <Location \"/_vti_cnf\">
    Order deny,allow
    Deny from all
  </Location>

<IfDefine HAVE_FRONTPAGE>
  ScriptAlias /_vti_bin/_vti_adm/ /var/www/html/_vti_b\in/_vti_adm/
  ScriptAlias /_vti_bin/_vti_aut/ /var/www/html/_vti_b\in/_vti_aut/
  ScriptAlias /_vti_bin/ /var/www/html/_vti_bin/
</IfDefine>

";
	    
	}


	print VHOSTFILE "</VirtualHost>\n";

	foreach my $ip (@ips) {
	    print VHOSTFILE "NameVirtualHost $ip\n";
	}

	close(VHOSTFILE);
    }
    
    # Read the information for this virtual host
    my $parser = new XML::DOM::Parser;
    my $doc = $parser->parsefile("$system{'datastore'}/virtualsites.xml");

    my $virtualSites = eval { $doc->getElementsByTagName("VirtualHost"); } || return(0);
    my $numOfVirtualSites = $virtualSites->getLength();

    # Find the right site
    my $siteNode = "";
    for (my $i=0; $i < $numOfVirtualSites; $i++) {

	my $thisSite = $virtualSites->item($i)->getElementsByTagName("ServerName")->item(0)->getFirstChild->getData();

	if ($thisSite eq $sitename) { 
	    $siteNode = $virtualSites->item($i);
	}
    }

    if ($siteNode eq "") {
	return(0);
    }
    else {

	# Parse information about log rotation from XML
	my $logparser = new XML::DOM::Parser;
	my $logdoc = $parser->parsefile("$system{'datastore'}/general.xml");

	# Grab the values for the log files
	my $weblog = eval { $logdoc->getElementsByTagName("Logs")->item(0)->getElementsByTagName("Weblog")->item(0)->getFirstChild->getData(); } || "";

	my $weblogfile = "
/var/log/httpd/access_log {
    missingok
    postrotate
        /bin/kill -HUP `cat /var/run/httpd.pid 2>/dev/null` 2> /dev/null || true
    endscript
}

/var/log/httpd/agent_log {
    missingok
    postrotate
        /bin/kill -HUP `cat /var/run/httpd.pid 2>/dev/null` 2> /dev/null || true
    endscript
}

/var/log/httpd/error_log {
    missingok
    postrotate
        /bin/kill -HUP `cat /var/run/httpd.pid 2>/dev/null` 2> /dev/null || true
    endscript
}

/var/log/httpd/referer_log {
    missingok
    postrotate
        /bin/kill -HUP `cat /var/run/httpd.pid 2>/dev/null` 2> /dev/null || true
    endscript
}
";

	# weblog
	if ($weblog eq "Once a Month") {
	    $weblogfile =~ s/missingok/missingok\n     monthly/g;
	}
	elsif ($weblog eq "When greater than 5 MB") {
	    $weblogfile =~ s/missingok/missingok\n     size=5M/g;
	}
	elsif ($weblog eq "When greater than 10 MB") {
	    $weblogfile =~ s/missingok/missingok\n     size=10M/g;
	}
	elsif ($weblog eq "When greater than 50 MB") {
	    $weblogfile =~ s/missingok/missingok\n     size=50M/g;
	}
	else {
	    $weblogfile =~ s/missingok/missingok\n     weekly/g;
	}
	
	# Grab information about the virtual site
	my $docroot = $siteNode->getElementsByTagName("DocumentRoot")->item(0)->getFirstChild->getData(); 
	my $siteadmin = $siteNode->getElementsByTagName("ServerAdmin")->item(0)->getFirstChild->getData();

	my $frontpage = eval { $siteNode->getElementsByTagName("AllowFrontPage")->item(0)->getFirstChild->getData(); } || "";
	
	my $servername = eval { $siteNode->getElementsByTagName("ServerName")->item(0)->getFirstChild->getData(); } || "";
	
	my $group = eval { $siteNode->getElementsByTagName("Group")->item(0)->getFirstChild->getData(); } || "";
	my $status = eval { $siteNode->getElementsByTagName("Status")->item(0)->getFirstChild->getData(); } || "";
	my $cgi = eval { $siteNode->getElementsByTagName("AllowCGI")->item(0)->getFirstChild->getData(); } || "";
	my $frontpage = eval { $siteNode->getElementsByTagName("AllowFrontPage")->item(0)->getFirstChild->getData(); } || "";
	my $php = eval { $siteNode->getElementsByTagName("AllowPHP")->item(0)->getFirstChild->getData(); } || "";
	my $modperl = eval { $siteNode->getElementsByTagName("AllowModPerl")->item(0)->getFirstChild->getData(); } || "";
	my $bandwidth = eval { $siteNode->getElementsByTagName("QOS")->item(0)->getElementsByTagName("Bandwidth")->item(0)->getFirstChild->getData(); } || "";
	my $limit_cpu = eval { $siteNode->getElementsByTagName("Limits")->item(0)->getElementsByTagName("CPU")->item(0)->getFirstChild->getData(); } || "";
	my $limit_memory = eval { $siteNode->getElementsByTagName("Limits")->item(0)->getElementsByTagName("Memory")->item(0)->getFirstChild->getData(); } || "";
	my $limit_procs = eval { $siteNode->getElementsByTagName("Limits")->item(0)->getElementsByTagName("Processes")->item(0)->getFirstChild->getData(); } || "";

	my $ip = eval { $siteNode->getAttribute("ip"); } || "";
	if ($ip eq "" || $ip eq "default") { $ip = "$iplist"; }

	# Grab some additional user information
	my %userInfo = datastore::userInfoByID($siteadmin);

	# Add the Virtual Site group
	my $groupID = addVSGroup($sitename, $sitename, $diskquota, @members);

	# Calculate primary directory
	my $directory = $docroot;
	$directory =~ s/\/html//;

	# Make symbolic links to user home directories
	foreach my $member (@members) {

	    my %memberInfo = datastore::userInfoByID($member);

	    # Make sym link for site
	    if ($memberInfo{'loginname'} ne "") {
		my $command = "/bin/ln -s $directory /home/$memberInfo{'loginname'}/$servername";
		my $result = `$command`;
		$command = "/bin/chown $memberInfo{'loginname'}:$memberInfo{'loginname'} /home/$memberInfo{'loginname'}/$servername";
		$result = `$command`;
		print STDERR "Result from sym link = $result\n";
	    }
	}

	my $command = "/bin/ln -s $directory /home/$userInfo{'loginname'}/$servername";
	my $result = `$command`;
	$command = "/bin/chown $userInfo{'loginname'}:$userInfo{'loginname'} /home/$userInfo{'loginname'}/$servername";
	$result = `$command`;
	
	# Update group node
      datastore::textNodeUpdate($siteNode->getElementsByTagName("Group"), $groupID);

	# Write the file back out
      datastore::XMLtoFile($doc,"$system{'datastore'}/virtualsites.xml");

	# Add site administrator to list of site_admins
	addUserToOSGroup($userInfo{'loginname'}, "site_admins");

	# Make primary directory
	$command = "/bin/mkdir $directory";
	$result = `$command`;

	# Make the document root directory
	$command = "/bin/mkdir $docroot";
	$result = `$command`;

	# Make the cgi-bin directory
	$command = "/bin/mkdir $directory/cgi-bin";
	$result = `$command`;

	# Make the mod_perl directory
	$command = "/bin/mkdir $directory/mod_perl";
	$result = `$command`;

	# Make the log directory
	$command = "/bin/mkdir $directory/logs";
	$result = `$command`;

	# Humor webalizer and create 5 access log files
	$command = "/bin/touch $directory/logs/access_log";
	$result = `$command`;
	$command = "/bin/chown root:root $directory/logs/error_log";
	$result = `$command`;
	$command = "/bin/chown root:root $directory/logs/access_log";
	$result = `$command`;

	# Make log rotation file for this virtual site
	$weblogfile =~ s/\/var\/log\/httpd\//$directory\/logs\//g;
	if (open(ROTATEFILE, ">/etc/logrotate.d/$sitename")) {

	    print ROTATEFILE $weblogfile;

	    close (ROTATEFILE);

	}

	# Setup webalizer
	if (open (FILE, ">$system{'root'}/system/reports/webalizer_$sitename.conf")) {

	    $command = "/bin/mkdir $directory/html/webalizer";
	    $result = `$command`;

	    print FILE "LogType clf\n";

	    print FILE "OutputDir     $directory/html/webalizer\n";

	    print FILE "HistoryName    webalizer.hist\n";

	    print FILE "HostName        $sitename\n\n";

	    print FILE "PageType	htm*\n";
	    print FILE "PageType	cgi\n";
	    print FILE "PageType	phtml\n";
	    print FILE "PageType	php3\n";
	    print FILE "PageType	pl\n\n";

	    print FILE "UseHTTPS        Yes\n";

	    print FILE "HTMLPre <!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0 Transitional//EN\">\n";
	    print FILE "HTMLHead <META NAME=\"Webalizer\" CONTENT=\"The Webalizer\">\n";
	    print FILE "HTMLBody <BODY BGCOLOR=\"#E8E8E8\" TEXT=\"#000000\" LINK=\"#0000FF\" VLINK=\"#FF0000\">\n";
	    print FILE "HTMLPost 	<BR CLEAR=\"all\">\n";
	    print FILE "HTMLEnd </BODY></HTML>\n";

	    print FILE "Quiet		no\n";
	    print FILE "ReallyQuiet	no\n";

	    print FILE "TimeMe		no\n";

	    print FILE "GMTTime		no\n";

	    print FILE "Debug		no\n";

	    print FILE "FoldSeqErr	no\n";

	    print FILE "VisitTimeout	1800\n";

	    print FILE "IgnoreHist	no\n";

	    print FILE "CountryGraph	yes\n";

	    print FILE "DailyGraph	yes\n";
	    print FILE "DailyStats	yes\n";

	    print FILE "HourlyGraph	yes\n";
	    print FILE "HourlyStats	yes\n";

	    print FILE "GraphLegend	yes\n";

	    print FILE "GraphLines	2\n";

	    print FILE "TopSites        30\n";
	    print FILE "TopKSites       10\n";
	    print FILE "TopURLs         30\n";
	    print FILE "TopKURLs        10\n";
	    print FILE "TopReferrers    30\n";
	    print FILE "TopAgents       15\n";
	    print FILE "TopCountries    30\n";
	    print FILE "TopEntry        10\n";
	    print FILE "TopExit         10\n";
	    print FILE "TopSearch       20\n";
	    print FILE "TopUsers        20\n";

	    print FILE "AllSites	no\n";
	    print FILE "AllURLs	        no\n";
	    print FILE "AllReferrers	no\n";
	    print FILE "AllAgents	no\n";
	    print FILE "AllSearchStr	no\n";
	    print FILE "AllUsers        no\n";

	    print FILE "IndexAlias     home.htm\n";
	    print FILE "IndexAlias	homepage.htm\n";

	    print FILE "HideURL		*.gif\n";
	    print FILE "HideURL		*.GIF\n";
	    print FILE "HideURL		*.jpg\n";
	    print FILE "HideURL		*.JPG\n";
	    print FILE "HideURL		*.png\n";
	    print FILE "HideURL		*.PNG\n";
	    print FILE "HideURL		*.ra\n";

	    print FILE "HideAllSites	no\n";

	    print FILE "GroupDomains	0\n";

	    print FILE "GroupShading	        yes\n";
	    print FILE "GroupHighlight	yes\n";

	    print FILE "MangleAgents    0\n";

	    print FILE "SearchEngine	yahoo.com	p=\n";
	    print FILE "SearchEngine	altavista.com	q=\n";
	    print FILE "SearchEngine	google.com	q=\n";
	    print FILE "SearchEngine	eureka.com	q=\n";
	    print FILE "SearchEngine	lycos.com	query=\n";
	    print FILE "SearchEngine	hotbot.com	MT=\n";
            print FILE "SearchEngine	msn.com		MT=\n";
	    print FILE "SearchEngine	infoseek.com	qt=\n";
	    print FILE "SearchEngine	webcrawler	searchText=\n";
	    print FILE "SearchEngine	excite		search=\n";
	    print FILE "SearchEngine	netscape.com	search=\n";
	    print FILE "SearchEngine	mamma.com	query=\n";
	    print FILE "SearchEngine	alltheweb.com	query=\n";
	    print FILE "SearchEngine	northernlight.com  qr=\n";

	    print FILE "DumpHeader	no\n";
	    print FILE "DumpExtension	tab\n";
	    print FILE "DumpSites	no\n";
	    print FILE "DumpURLs	no\n";
	    print FILE "DumpReferrers	no\n";
	    print FILE "DumpAgents	no\n";
	    print FILE "DumpUsers	no\n";
	    print FILE "DumpSearchStr  no\n";
		     
	    close(FILE);

	    # Change ownership permissions
	    $command = "/bin/chown -R $siteadmin:$groupID $docroot/webalizer";
	    $result = `$command`;

	}

	# Place a temporary index file in the site directory
	if (open (FILE, ">$docroot/index.html")) {

	    print FILE "<HTML>
           <BODY bgcolor=white>
              <H2><CENTER>This is the future site for \"$sitename\".</CENTER></H2>
           </BODY>
           </HTML>";

	    close (FILE);
	}

	# Make a place holder for the webalizer report
	open (FILE, ">$docroot/webalizer/index.html");
	print FILE "<HTML>
           <BODY bgcolor=white>
              <H2><CENTER>Web server statistics have not yet run for \"$sitename\".  When they have run, this report will graphically illustrate your site usage.</CENTER></H2>
           </BODY>
           </HTML>";
	close (FILE);

	# Add the Virtual Site to the vhosts.conf file
	if (open(VHOSTFILE, "<$system{'vhostsfile'}")) {

	    my @lines = <VHOSTFILE>;

	    close(VHOSTFILE);

	    if (open(VHOSTFILE, ">$system{'vhostsfile'}")) {

		# Print all existing configurations
		foreach my $line (@lines) {
		    print VHOSTFILE $line;
		}

		# Add the new one if active
		if ($status eq "Active") {
		
		    print VHOSTFILE "
<VirtualHost $ip>
  ServerName $servername
  CustomLog $directory/logs/access_log common
  LogLevel warn
  ErrorLog $directory/logs/error_log
  DocumentRoot $docroot

  <Directory $docroot>
    AllowOverride All
  </Directory>
";

		    # Setup basic cgi
		    if ($cgi eq "Yes") {

			print VHOSTFILE "
  ScriptAlias /cgi-bin/ \"$directory/cgi-bin/\"

  <Directory \"$directory/cgi-bin\">
    AllowOverride None
    Options ExecCGI
    Order allow,deny
    Allow from all
  </Directory>

";
		    }

		    # Deny access to php served pages
		    if ($php ne "Yes") {
			print VHOSTFILE "<Files ~ \"\\.*(php3|php4|php|phtml)\">
   Order allow,deny
   Deny from all
</Files>

";
		    }


		    # Enabling/Disabling mod_perl
		    if ($modperl eq "Yes") {

			# Setup mod_perl
			print VHOSTFILE "
<IfModule mod_perl.c>

  ScriptAlias /mod_perl/ \"$directory/mod_perl/\"

  <Location /mod_perl>
    SetHandler perl-script
    PerlHandler Apache::Registry
    Options +ExecCGI
  </Location>
</IfModule>

";

		}

		    # Setup mod_throttle
		    if ($bandwidth ne "" && $bandwidth ne "0") {
#		    print VHOSTFILE "
# ThrottlePolicy Volume $bandwidth 1s\n";
		    }

		    # Setup DoS Limits
		    if ($limit_cpu ne "") {
			print VHOSTFILE "
  RLimitCPU     $limit_cpu";
		    }
		    if ($limit_procs ne "") {
			print VHOSTFILE "
  RLimitNPROC     $limit_procs";
		    }
		    if ($limit_memory ne "") {
			print VHOSTFILE "
  RLimitMEM     $limit_memory";
		    }
		    
		    # Setup Front Page Extensions
		    print VHOSTFILE "
  <Location \"/_vti_pvt\">
    Order deny,allow
    Deny from all
  </Location>
  <Location \"/_vti_cnf\">
    Order deny,allow
    Deny from all
  </Location>
";

		    if ($frontpage eq "Yes") {

			print VHOSTFILE "
<IfDefine HAVE_FRONTPAGE>
  ScriptAlias /_vti_bin/_vti_adm/ /home/VirtualSites/$servername/html/_vti_b\in/_vti_adm/
  ScriptAlias /_vti_bin/_vti_aut/ /home/VirtualSites/$servername/html/_vti_b\in/_vti_aut/
  ScriptAlias /_vti_bin/ /home/VirtualSites/$servername/html/_vti_bin/
</IfDefine>
";

		    }
		    else {
			print VHOSTFILE "
  <Location \"/_vti_bin\">
    Order deny,allow
    Deny from all
  </Location>
";
		    }

		    print VHOSTFILE "
</VirtualHost>\n";


		    # Update permissions on the directories
		    my $command = "/bin/chown $siteadmin:$group $docroot";
		    my $result = `$command`;

		}

	    }

	    close(VHOSTFILE);
	}

	my $dirs = `/bin/ls $directory/html/*`;
	print STDERR "Before running fpsrvadmin $dirs\n";

	# Make Front Page happy
	if (open(FPFILE, ">/etc/httpd/conf/fphttpd.conf")) {

	    my $command = "/bin/cat $system{'httpd.conf'}";
	    my $result = `$command`;

	    print FPFILE "$result\n";

	    $command = "/bin/cat /etc/httpd/conf/vhosts.conf";
	    $result = `$command`;

	    print FPFILE "$result\n";

	    close(FPFILE);

	    $command = "/bin/chmod o+r /etc/httpd/conf/fphttpd.conf";
	    $result = `$command`;

	}

	print STDERR "Building Front Page Directories\n";

	# Make FP directories
	$command = "echo \"0\n$userInfo{'loginname'}\n$groupID\n$userInfo{'loginname'}\ntemporary442\ntemporary442\n\" | /usr/local/frontpage/version4.0/bin/fpsrvadm_suid.exe -operation install -port 80 -multihost $sitename -servconf /etc/httpd/conf/fphttpd.conf";
	# WARNING:  Do not remove this STDERR!!
	print STDERR "Command for building front page = $command\n";
	$result = `$command`;
	
	print STDERR "FRONTPAGE created with $command\n\n Results = $result\n";

	my $result = `/bin/rm $docroot/_vti_pvt/service.pwd`;
	$result = `/bin/rm $docroot/_vti_pvt/service.grp`;

	# Update the password files
	# Rebuild the service.grp file
	if (open (FPFILE, ">$docroot/_vti_pvt/service.grp")) {
	    print FPFILE "# -FrontPage-\n";
	    print FPFILE "administrators: $userInfo{'loginname'}\n";
	    
	    print FPFILE "authors: ";
	    
	    close(FPFILE);
	}
	
	# Rebuild the service.pwd file
	if (open (FPFILE, ">$docroot/_vti_pvt/service.pwd")) {
	    print FPFILE "# -FrontPage-\n";
	    close(FPFILE);
	}
	
	updateHTPasswd ($userInfo{'loginname'}, "$docroot/_vti_pvt/service.pwd");

	# Update .htaccess to play nicely with autoindex
	if (open(HTACCESS, "$docroot/.htaccess")) {

	    my @htlines = <HTACCESS>;

	    close(HTACCESS);

	    if (open(HTACCESS, ">$docroot/.htaccess")) {

		foreach my $htline (@htlines) {

		    if ($htline =~ m/IndexIgnore/) {
			$htline = "<IfModule mod_autoindex.c>
  IndexIgnore .htaccess */.??* *~ *# */HEADER* */README* */_vti*
</IfModule>\n";
		    }

		    print HTACCESS $htline;
		}

		close(HTACCESS);
	    }
	}
	
	print STDERR "Changing ownership\n";

	# Change ownership permissions
	$command = "/bin/chown $siteadmin:$groupID $docroot/index.html";
	$result = `$command`;

	$command = "/bin/chown -R $siteadmin:$groupID $directory/html/_vti_pvt/service.pwd";
	$result = `$command`;	
	$command = "/bin/chown -R $siteadmin:$groupID $directory/html/_vti_pvt/service.grp";
	$result = `$command`;	

	# Fix permissions on lock file
	$command = "/bin/chmod o-wx $directory/html/_vti_pvt/frontpg.lck";
	$result = `$command`;	

	$command = "/bin/chown -R $siteadmin:$groupID $directory";
	$result = `$command`;
	$command = "/bin/chown -R apache:$groupID $directory";
	$result = `$command`;
	$command = "/bin/chmod g+ws $directory";
	$result = `$command`;
	$command = "/bin/chmod o-rwx $directory";
	$result = `$command`;
	$command = "/bin/chown -R $siteadmin:$groupID $directory/html";
	$result = `$command`;
	$command = "/bin/chmod g+ws $directory/html";
	$result = `$command`;
	$command = "/bin/chown -R $siteadmin:$groupID $directory/logs";
	$result = `$command`;
	$command = "/bin/chmod o-rwx $directory/logs";
	$result = `$command`;
	$command = "/bin/chown -R $siteadmin:$groupID $directory/cgi-bin";
	$result = `$command`;
	$command = "/bin/chmod g+ws $directory/cgi-bin";
	$result = `$command`;
	$command = "/bin/chown -R $siteadmin:$groupID $directory/mod_perl";
	$result = `$command`;
	$command = "/bin/chmod g+ws $directory/mod_perl";
	$result = `$command`;


	$command = "$system{'root'}/system/bin/chmod ug+ws $docroot/_vti_bin/shtml.exe";
	$result = `$command`;
	$command = "$system{'root'}/system/bin/chmod ug+ws $docroot/_vti_bin/fpcount.exe";
	$result = `$command`;
	$command = "$system{'root'}/system/bin/chmod ug+ws $docroot/_vti_bin/_vti_adm/admin.exe";
	$result = `$command`;
	$command = "$system{'root'}/system/bin/chmod ug+ws $docroot/_vti_bin/_vti_aut/author.exe";
	$result = `$command`;

	# Update the sendmail generics domain
	updateSendmailGenericsDomain();

	print STDERR "Ownership updated\n";

	print STDER "Calling update sendmail\n";
	updateSendmail();

	# Restart Apache
	stopApache();
	restartApache();

    }

    return(1);
}

# A sub-routine that deletes the virtual site from the system
sub delVirtualSite {

    my %system = config::system();

    my $sitename = shift || return(0);

    my $groupID = "";
    my $siteadmin = "";
    
    # Remove the site from the XML
    # Delete it from the XML
    # Parse the virtual sites file
    my $parser = new XML::DOM::Parser;
    my $doc = $parser->parsefile("$system{'datastore'}/virtualsites.xml");

    my $virtualSites = eval { $doc->getElementsByTagName("VirtualHost"); } || "";
    my $numOfVirtualSites = eval { $virtualSites->getLength(); } || "0";
    
    # Find the correct virtual site
    for (my $i=0; $i<$numOfVirtualSites; $i++) {
	
        my $thisServerName = $virtualSites->item($i)->getElementsByTagName("ServerName")->item(0)->getFirstChild->getData();
	
	# If we found the right site, delete it
	if ($sitename eq $thisServerName) {

	    # Now remove the virtual site from the datastore
	    $groupID = eval { $virtualSites->item($i)->getElementsByTagName("Group")->item(0)->getFirstChild->getData(); } || "";
	    $siteadmin = eval { $virtualSites->item($i)->getElementsByTagName("ServerAdmin")->item(0)->getFirstChild->getData(); } || "";
            $doc->getFirstChild->removeChild($virtualSites->item($i));
            last;
	}
    }

    print STDERR "About to delete virtual site group virtualsite_$groupID\n";

    my %userInfo = datastore::userInfoByID($siteadmin);
    delUserFromOSGroup($userInfo{'loginname'}, "site_admins");

    # Parse the virtual sites file
    my $parser2 = new XML::DOM::Parser;
    my $grpdoc = $parser2->parsefile("$system{'datastore'}/groups.xml");

    # Remove symbolic links
    my $command = "/bin/cat /etc/group | /bin/grep ^virtualsite_$groupID";
    my $result = `$command`;
    $result =~ s/\n//g;

    my @pieces = split(":", $result);
    my @members = split(",", $pieces[3]);
    
    foreach my $member (@members) {
	$command = "/bin/rm /home/$member/$sitename";
	$result = `$command`;

	print STDERR "Command = $command\nResult = $result\n";
    }

    # Remove sym link for primary administrator
    my $command = "/bin/rm /home/$userInfo{'loginname'}/$sitename";
    my $result = `$command`;

  datastore::delGroup($grpdoc, "virtualsite_$groupID");

  datastore::XMLtoFile($grpdoc, "$system{'datastore'}/groups.xml");

  datastore::XMLtoFile($doc, "$system{'datastore'}/virtualsites.xml");

    # Remove the virtual site group
    $command = "/usr/sbin/groupdel virtualsite_$groupID";
    $result = `$command`;

    $command = "/usr/sbin/grpconv";
    $result = `$command`;

    # Remove the directory
    # This is extreme prejudice...need to add options for backing up
    $command = "/bin/rm -rf $system{'vhostroot'}/$sitename";
    $result = `$command`;

    $command = "/bin/rm /etc/logrotate.d/$sitename";
    $result = `$command`;

    $command = "/bin/rm /usr/local/frontpage/$sitename:80.cnf";
    $result = `$command`;

    $command = "/bin/rm $system{'root'}/system/reports/webalizer_$sitename.conf";
    $result = `$command`;

    # Remove the virtual site from the vhost.conf file
    if (open(VHOSTFILE, "<$system{'vhostsfile'}")) {

	my @lines = <VHOSTFILE>;

	close(VHOSTFILE);

	my $inVirtualHost = 0;
	my @newlines = ();

	# Strip out this virtual site
	foreach my $line (@lines) {
	    
	    if ($line =~ m/ServerName $sitename/) {
		pop(@newlines);
		$inVirtualHost = 1;
	    }
	    elsif ($line =~ m/<\/VirtualHost>/ && $inVirtualHost == 1) {
		$inVirtualHost = 0;
		next;
	    }
	    elsif ($line =~ m/<VirtualHost/ && $inVirtualHost == 1) {
		push(@newlines,$line);
		$inVirtualHost = 0;
	    }
	    elsif ($line =~ m/ServerName/ && $inVirtualHost == 1) {
		push(@newlines,$line);
		$inVirtualHost = 0;
	    }
	    elsif ($inVirtualHost == 1) {
		next;
	    }
	    else {
		push(@newlines, $line);
	    }
	    
	}

	if (open(VHOSTFILE, ">$system{'vhostsfile'}")) {

	    foreach my $line (@newlines) {
		print VHOSTFILE $line;
	    }

	    close(VHOSTFILE);

	}

    }

    # Update sendmail generics domain
    updateSendmailGenericsDomain();

    # Update the e-mail aliases
    updateSendmail();

    # Restart Apache
    restartApache();

}

# A sub-routine that updates the IP address in vhosts.conf
sub updateVirtualSiteIPs {

    my %system = config::system;

    print STDERR "Updating virtual site IPs\n";
    
    # First grab all IP addresses
    my $query = 'WHERE
	    <Interface>
	      <IP>$ip</IP>
	    </Interface>
    	    IN "file:' . $system{'datastore'} . '/network.xml"
      	    CONSTRUCT $ip<TD>';

    # Make the query
    my $result = XML::QL->query($query);
    
    # Split out the values
    my @ips = split(/<TD>/,$result);
    my $iplist = join(' ', @ips);

    # If the vhosts file does not exists, populate it with the root web
    if  ((-s $system{'vhostsfile'}) <= 0) {

	`/bin/rm /usr/local/frontpage/we80.cnf`;

	open(VHOSTFILE, ">$system{'vhostsfile'}");

	print VHOSTFILE "<VirtualHost $iplist>
   DocumentRoot \"/var/www/html\"\n";

	my $fpextensions = `/bin/cat $system{'datastore'}/apache.xml | /bin/grep \\\<FrontPageExtensions\\\>Yes`;

	# If front page is on, add it to the root web
	if ($fpextensions ne "") {
	    
	    print VHOSTFILE "
  <Directory /var/www/html>
    AllowOverride All
  </Directory>

  <Location \"/_vti_pvt\">
    Order deny,allow
    Deny from all
  </Location>
  <Location \"/_vti_cnf\">
    Order deny,allow
    Deny from all
  </Location>

<IfDefine HAVE_FRONTPAGE>
  ScriptAlias /_vti_bin/_vti_adm/ /var/www/html/_vti_b\in/_vti_adm/
  ScriptAlias /_vti_bin/_vti_aut/ /var/www/html/_vti_b\in/_vti_aut/
  ScriptAlias /_vti_bin/ /var/www/html/_vti_bin/
</IfDefine>

";

	}


	print VHOSTFILE "</VirtualHost>\n";

	foreach my $ip (@ips) {
	    print VHOSTFILE "NameVirtualHost $ip\n";
	}

	close(VHOSTFILE);

    }

    # Read in the current vhosts.conf file
    if (open(VHOSTFILE, "<$system{'vhostsfile'}")) {
	
	my @lines = <VHOSTFILE>;
	close(VHOSTFILE);
	
	# Overwrite the current virtual sites file
	if (open(VHOSTFILE, ">$system{'vhostsfile'}")) {
	    
	    my $printedNameVirtualHost = 0;

	    foreach my $line (@lines) {

		if ($line =~ m/^NameVirtualHost/ && $printedNameVirtualHost == 0) {
		    $printedNameVirtualHost = 1;
		    foreach my $ip (@ips) {
			print VHOSTFILE "NameVirtualHost $ip\n";
		    }
		}
		elsif ($line =~ m/^NameVirtualHost/) {
		    next;
		}
		elsif ($line =~ m/^<VirtualHost (.*) (.*)>/) {
		    print VHOSTFILE "<VirtualHost $iplist>\n";
		}
		#  NOTE:  FIX THIS!!!!
		elsif ($line =~ m/<VirtualHost (.*)>/) {
		    print VHOSTFILE "<VirtualHost $iplist>\n";
		}
		else {
		    print VHOSTFILE $line;
		}
	    }
	}
    }
	
    return(1);

}

# A sub-routine that updates the virtual host entries
sub updateVirtualSite {

    my $sitename = shift || "";
    my $oldsiteadmin = shift || "";

#    print STDERR "    RECEIVED OLD SITE ADMIN OF $oldsiteadmin\n";

    my %system = config::system;

    # First grab all IP addresses
    my $query = 'WHERE
	    <Interface>
	      <IP>$ip</IP>
	    </Interface>
    	    IN "file:' . $system{'datastore'} . '/network.xml"
      	    CONSTRUCT $ip<TD>';

    # Make the query
    my $result = XML::QL->query($query);
    
    # Split out the values
    my @ips = split(/<TD>/,$result);
    my $iplist = join(' ', @ips);
    print STDERR "Updating virtual host file\n";

    my $newVhostFile = "";

    # Parse the virtual sites
    my $parser = new XML::DOM::Parser;
    my $doc = $parser->parsefile("$system{'datastore'}/virtualsites.xml");
    
    # Search for this virtual site
    my $foundNode = "";
    for my $node ($doc->getElementsByTagName("VirtualHost")) {

	my $thisSiteName = eval { $node->getElementsByTagName("ServerName")->item(0)->getFirstChild->getData(); } || "";

	if ($thisSiteName eq $sitename) {
	    $foundNode = $node;
	    last;
	}
    }

    if ($foundNode eq "") {
	print STDERR "Attempted virtual site update on non-existant site!\n";
	return(1);
    }
    else {


	my $docroot = $foundNode->getElementsByTagName("DocumentRoot")->item(0)->getFirstChild->getData(); 
	my $directory = $docroot;
	$directory =~ s/\/html//;

	my $siteadmin = $foundNode->getElementsByTagName("ServerAdmin")->item(0)->getFirstChild->getData();

	my %adminInfo = datastore::userInfoByID($siteadmin);
	my %oldAdminInfo = datastore::userInfoByID($oldsiteadmin);

	print STDERR "  OLDSITE ADMIN LOGINNAME = $oldAdminInfo{'loginname'}\n";

	# Update the site_admins group here

#	print STDERR "   **** Updating site admins: deleting $oldAdminInfo{'loginname'} and adding $adminInfo{'loginname'}\n";

	delUserFromOSGroup($oldAdminInfo{'loginname'}, "site_admins");
	addUserToOSGroup($adminInfo{'loginname'}, "site_admins");

	my $frontpage = eval { $foundNode->getElementsByTagName("AllowFrontPage")->item(0)->getFirstChild->getData(); } || "";	
	my $group = eval { $foundNode->getElementsByTagName("Group")->item(0)->getFirstChild->getData(); } || "";
	my $status = eval { $foundNode->getElementsByTagName("Status")->item(0)->getFirstChild->getData(); } || "";
	my $cgi = eval { $foundNode->getElementsByTagName("AllowCGI")->item(0)->getFirstChild->getData(); } || "";
	my $frontpage = eval { $foundNode->getElementsByTagName("AllowFrontPage")->item(0)->getFirstChild->getData(); } || "";
	my $php = eval { $foundNode->getElementsByTagName("AllowPHP")->item(0)->getFirstChild->getData(); } || "";
	my $modperl = eval { $foundNode->getElementsByTagName("AllowModPerl")->item(0)->getFirstChild->getData(); } || "";
	my $bandwidth = eval { $foundNode->getElementsByTagName("QOS")->item(0)->getElementsByTagName("Bandwidth")->item(0)->getFirstChild->getData(); } || "";
	my $limit_cpu = eval { $foundNode->getElementsByTagName("Limits")->item(0)->getElementsByTagName("CPU")->item(0)->getFirstChild->getData(); } || "";
	my $limit_memory = eval { $foundNode->getElementsByTagName("Limits")->item(0)->getElementsByTagName("Memory")->item(0)->getFirstChild->getData(); } || "";
	my $limit_procs = eval { $foundNode->getElementsByTagName("Limits")->item(0)->getElementsByTagName("Processes")->item(0)->getFirstChild->getData(); } || "";

	# Make sure the site administrator is a part of the virtual site group
	addUserToOSGroup($adminInfo{'loginname'}, "virtualsite_$group");

	my $ip = eval { $foundNode->getAttribute("ip"); } || "";
	if ($ip eq "" || $ip eq "default") { $ip = "$iplist"; }

	# Remove the virtual site from the vhost.conf file
	if (open(VHOSTFILE, "<$system{'vhostsfile'}")) {
	    
	    my @lines = <VHOSTFILE>;
	    
	    close(VHOSTFILE);
	    
	    my $inVirtualHost = 0;
	    my @newlines = ();
	    
	    # Strip out this virtual site
	    foreach my $line (@lines) {
		
		if ($line =~ m/ServerName $sitename/) {
		    pop(@newlines);
		    $inVirtualHost = 1;
		}
		elsif ($line =~ m/<\/VirtualHost>/ && $inVirtualHost == 1) {
		    $inVirtualHost = 0;
		    next;
		}
		elsif ($line =~ m/<VirtualHost/ && $inVirtualHost == 1) {
		    push(@newlines,$line);
		    $inVirtualHost = 0;
		}
		elsif ($line =~ m/ServerName/ && $inVirtualHost == 1) {
		    push(@newlines,$line);
		    $inVirtualHost = 0;
		}
		elsif ($inVirtualHost == 1) {
		    next;
		}
		else {
		    push(@newlines, $line);
		}
	    
	    }
	    
	    if (open(VHOSTFILE, ">$system{'vhostsfile'}")) {
		
		foreach my $line (@newlines) {
		    print VHOSTFILE $line;
		}

		# Add the new one if active
		if ($status eq "Active") {
		    
		    print VHOSTFILE "
<VirtualHost $ip>
  ServerName $sitename
  CustomLog $directory/logs/access_log common
  LogLevel warn
  ErrorLog $directory/logs/error_log
  DocumentRoot $docroot

  <Directory $docroot>
    AllowOverride All
  </Directory>
";

		    # Setup basic cgi
		    if ($cgi eq "Yes") {

			print VHOSTFILE "
  ScriptAlias /cgi-bin/ \"$directory/cgi-bin/\"

  <Directory \"$directory/cgi-bin\">
    AllowOverride None
    Options ExecCGI
    Order allow,deny
    Allow from all
  </Directory>

";
		    }

		    # Deny access to php served pages
		    if ($php ne "Yes") {
			print VHOSTFILE "<Files ~ \"\\.*(php3|php4|php|phtml)\">
   Order allow,deny
   Deny from all
</Files>

";
		    }

		    if ($modperl eq "Yes") {

			# Setup mod_perl
			print VHOSTFILE "

<IfModule mod_perl.c>
  ScriptAlias /mod_perl/ \"$directory/mod_perl/\"

  <Location /mod_perl>
    SetHandler perl-script
    PerlHandler Apache::Registry
    Options +ExecCGI
  </Location>
</IfModule>
";
		    }

		    # Setup mod_throttle
		    if ($bandwidth ne "" && $bandwidth ne "0") {
#		    print VHOSTFILE "
# ThrottlePolicy Volume $bandwidth 1s\n";
		    }

		    # Setup DoS Limits
		    if ($limit_cpu ne "") {
			print VHOSTFILE "
  RLimitCPU     $limit_cpu";
		    }
		    if ($limit_procs ne "") {
			print VHOSTFILE "
  RLimitNPROC     $limit_procs";
		    }
		    if ($limit_memory ne "") {
			print VHOSTFILE "
  RLimitMEM     $limit_memory";
		    }
		    
		    # Setup Front Page Extensions
		    print VHOSTFILE "
  <Location \"/_vti_pvt\">
    Order deny,allow
    Deny from all
  </Location>
  <Location \"/_vti_cnf\">
    Order deny,allow
    Deny from all
  </Location>
";

		    if ($frontpage eq "Yes") {

			print VHOSTFILE "

<IfDefine HAVE_FRONTPAGE>
  ScriptAlias /_vti_bin/_vti_adm/ /home/VirtualSites/$sitename/html/_vti_b\in/_vti_adm/
  ScriptAlias /_vti_bin/_vti_aut/ /home/VirtualSites/$sitename/html/_vti_b\in/_vti_aut/
  ScriptAlias /_vti_bin/ /home/VirtualSites/$sitename/html/_vti_bin/
</IfDefine>
";

		    }
		    else {
			print VHOSTFILE "
  <Location \"/_vti_bin\">
    Order deny,allow
    Deny from all
  </Location>
";
		    }

		    print VHOSTFILE "
</VirtualHost>\n";

		}

	    }

	    close(VHOSTFILE);
	    
	}

	# Make sure the site administrator is a part of the virtual site group
	addUserToOSGroup($siteadmin, "virtualsite_$group");

	# Reset permissions
	my $command = "/bin/chown -R apache:$group $directory";
	my $result = `$command`;
	$command = "/bin/chmod g+ws $directory";
	$result = `$command`;
	$command = "/bin/chown -R $siteadmin:$group $directory/html";
	$result = `$command`;
	$command = "/bin/chmod g+ws $directory/html";
	$result = `$command`;
	$command = "/bin/chown -R $siteadmin:$group $directory/logs";
	$result = `$command`;	
	$command = "/bin/chmod o-rwx $directory/logs";
	$result = `$command`;
	$command = "/bin/chown -R $siteadmin:$group $directory/cgi-bin";
	$result = `$command`;
	$command = "/bin/chmod g+ws $directory/cgi-bin";
	$result = `$command`;
	$command = "/bin/chown -R $siteadmin:$group $directory/mod_perl";
	$result = `$command`;	
	$command = "/bin/chmod g+ws $directory/mod_perl";
	$result = `$command`;

	# Fix up front page permissions
	$command = "$system{'root'}/system/bin/chmod ug+ws $docroot/_vti_bin/shtml.exe";
	$result = `$command`;
	$command = "$system{'root'}/system/bin/chmod ug+ws $docroot/_vti_bin/fpcount.exe";
	$result = `$command`;
	$command = "$system{'root'}/system/bin/chmod ug+ws $docroot/_vti_bin/_vti_adm/admin.exe";
	$result = `$command`;
	$command = "$system{'root'}/system/bin/chmod ug+ws $docroot/_vti_bin/_vti_aut/author.exe";
	$result = `$command`;

	# Update Front Page password file
	$result = `/bin/rm $docroot/_vti_pvt/service.pwd`;
	$result = `/bin/rm $docroot/_vti_pvt/service.grp`;

        my %userInfo = datastore::userInfoByID($siteadmin);

        # Update the password files
	# Rebuild the service.grp file
        if (open (FPFILE, ">$docroot/_vti_pvt/service.grp")) {
            print FPFILE "# -FrontPage-\n";
            print FPFILE "administrators: $userInfo{'loginname'}\n";

            print FPFILE "authors: ";

            close(FPFILE);
        }

	# Rebuild the service.pwd file
        if (open (FPFILE, ">$docroot/_vti_pvt/service.pwd")) {
            print FPFILE "# -FrontPage-\n";
	    close(FPFILE);
	}

	updateHTPasswd ($userInfo{'loginname'}, "$docroot/_vti_pvt/service.pwd");

	# Fix-up permissions
	$command = "/bin/chown -R $siteadmin:$group $directory/html/_vti_pvt/service.pwd";
	$result = `$command`;	
	$command = "/bin/chown -R $siteadmin:$group $directory/html/_vti_pvt/service.grp";
	$result = `$command`;

	# Add symbolic links
	my $members = `/bin/cat /etc/group | /bin/grep ^virtualsite_$group`;
	my @pieces = split(":", $members);
	my @members = split(",", $pieces[3]);

	foreach my $member (@members) {

	    $member =~ s/\n//g;

	    $result = `/bin/ln -s $directory /home/$member/$sitename`;
	    $result = `/bin/chown $member:$member /home/$member/$sitename`;

	}

    }

    print STDERR "Returning from updateVirtualSite\n";

    return(1);

}

# A sub-routine that applies a profile to the apache configuration
sub applyApacheProfile {

    # Step 1:  Grab profile id and all applied items
    my $profileid = shift || return;
    my $itemsOn = join(' ', @_);
    $itemsOn = " $itemsOn ";

    my %system = config::system();

    # Step 1a:  Update the general.xml to store selected profile
    my $genparser = new XML::DOM::Parser;
    my $gendoc = $genparser->parsefile("$system{'datastore'}/general.xml");

  datastore::textNodeUpdate($gendoc->getElementsByTagName("SystemProfile")->item(0), $profileid);

    $gendoc->printToFile("$system{'datastore'}/general.xml");

    $gendoc->dispose();

    # Step 2:  Read profiles information and get the detail for this one
    my $parser = new XML::DOM::Parser;
    my $doc = $parser->parsefile("$system{'datastore'}/profiles.xml");

    my $foundNode = "";
    foreach my $node ($doc->getElementsByTagName("Profile")) {

	my $thisid = $node->getAttribute("id");

	if ($thisid eq $profileid) {
	    $foundNode = $node;
	    last;
	}
    }

    if ($foundNode eq "") {
	$doc->dispose();
	return(0);
    }

    # Step 3:  Update apache.xml
    my $items = $foundNode->getElementsByTagName("Item");
    my $numOfItems = $items->getLength();

    my $apacheDaemon = "Standard";

    print STDERR "Number of Items = $numOfItems\n";

    for (my $k=0; $k<$numOfItems; $k++) {

	my $node = $items->item($k);
	my $itemid = eval { $node->getAttribute("id"); } || "";

#	print STDERR "Item ID = $itemid\n";

	if ($itemid <= 0) {
	    
	    $apacheDaemon = eval { $node->getElementsByTagName("Apache")->item(0)->getFirstChild->getData(); } || "Standard";
	    print STDERR "Skipping sub zero item detail\n";
	    next;
	}

	# If this item is on, then make the change
	if ($itemsOn =~ m/ $itemid /) {

	    foreach my $fileNode ($node->getElementsByTagName("File")) {

		my $name = $fileNode->getAttribute("name");

#		print STDERR "FILE = $name\n";

		my $parser2 = new XML::DOM::Parser;
		my $doc2 = $parser->parsefile("$system{'datastore'}/$name");

		# Step 3a:  For each node in the detail, update the actual file
		# WARNING:  This is not generally applicable as a DOM tree merge.  It works
		# on the assumption that tags are strictly unique within the document.
		# Don't we wish everything was made like rubbermaid?  :-)
		foreach my $change ($fileNode->getChildNodes()) {

		    my @children = $change->getChildNodes();
		    my $numOfChildren = scalar(@children);

		    for (my $i=0; $i<$numOfChildren; $i++) {
			
			my $changeTag = eval { $children[$i]->getNodeName(); } || "";
#			print STDERR "$i) $itemid in file $name Found change node $changeTag\n";

			if ($changeTag ne "#text#") {
			    my $changeValue = eval { $children[$i]->getFirstChild->getData(); } || "";
			    print STDERR "  - Change value = $changeValue\n";
			  datastore::textNodeUpdate($doc2->getElementsByTagName("$changeTag")->item(0), $changeValue);
			}
		    }
		}

		$doc2->printToFile("$system{'datastore'}/$name");

		$doc2->dispose();

	    }
	}

    }

    # Step 4:  Move httpd daemon (if necessary), DSO's, and config files
    stopApache();

    # Give stop apache some time to work
    sleep(3);

    # Give the web server time to die
    my $httpdresult = `/bin/ps auwx | /bin/grep /usr/sbin/httpd | /usr/bin/wc -l`;
    $httpdresult =~ s/\W//g;
    $httpdresult = scalar($httpdresult);

    print STDERR "HTTPDRESULTS = $httpdresult\n";

    my $count = 0;

    while ($httpdresult > 3 && $count < 6) {
	++$count;
	sleep(1);
	$httpdresult = `/bin/ps auwx | /bin/grep /usr/sbin/httpd | /usr/bin/wc -l`;
	$httpdresult =~ s/\W//g;
	$httpdresult = scalar($httpdresult);
	print STDERR "Sleeping 1 second - $httpdresult\n";
    }

    print STDERR "Daemon = $apacheDaemon\n";
    if ($apacheDaemon eq "Standard") {
	print STDERR `/bin/cp /usr/compaq/bin/apache/apachestd/httpd /usr/sbin/httpd`;
    }
    elsif ($apacheDaemon eq "Optimized") {
	print STDERR `/bin/cp /usr/compaq/bin/apache/apacheopt/httpd /usr/sbin/httpd`;
    }

    # Step 5:  Cycle Apache
    updateApache();

}

# A sub-routine that updates the httpd.conf file based on the XML configuration
sub updateApache {

    my %system = config::system;

    # Release 16 - This is a patch to correct fphttpd.conf
    if ( -e "$system{'root'}/system/fphttpd.conf" ) {
	updateVirtualSitesFPConf();
	`/bin/rm $system{'root'}/system/fphttpd.conf`;
    }



    my $configFile = $system{'httpd.conf'};

    # Parse the apache file
    my $parser = new XML::DOM::Parser;
    my $apachedoc = $parser->parsefile("$system{'datastore'}/apache.xml");

    # O.K. Step #1 - Read existing httpd.conf
    open(FILE, "<$configFile");
    my @lines = <FILE>;
    my $numOfLines = scalar(@lines);
    close(FILE);

    # Step #2 - General, Global Configuration Information
    for (my $i=0; $i<$numOfLines; $i++) {

	# Skip things that break this logic
	if ($lines[$i] =~ m/mod_foo.so/) {
	    next;
	}

	# Load/Un-load modules
	if ($lines[$i] =~ m/^LoadModule/) {

  	    # Extract module name using split
  	    my @mods = split(" ", $lines[$i]);
           
  	    # Check if it exists in the XML
	    my $checkXML = datastore::ApacheCheckModule($apachedoc,$mods[1]);

  	    # If it does exist in XML, then leave the line alone
  	    # If it does not exist in XML, then comment it out
  	    if ($checkXML != 1) {
 		$lines[$i] =~ s/^\s*LoadModule/# LoadModule/;
  	    }

	    next;

  	}
  	elsif ($lines[$i] =~ m/^\#\s*LoadModule/) {
	    
  	    # Extract module name using split
  	    my @mods = split(" ", $lines[$i]);
	    
  	    # Check if it exists in the XML
  	    my $checkXML = datastore::ApacheCheckModule($apachedoc,$mods[2]);
	    
  	    # If it does exist in XML, then uncomment line
  	    # If it does not exist in XML, leave it alone
	    
  	    if ($checkXML == 1) {
  		$lines[$i] =~ s/^#\s*LoadModule/LoadModule/;
  	    }

	    next;

  	}
	# Check for corresponding add module directive
	elsif ($lines[$i] =~ m/^AddModule/) {

  	    # Extract module name using split
  	    my @mods = split(" ", $lines[$i]);
           
  	    # Check if it exists in the XML
	    my $checkXML = datastore::ApacheCheckModule($apachedoc,$mods[1]);
           
  	    # If it does exist in XML, then leave the line alone
  	    # If it does not exist in XML, then comment it out
  	    if ($checkXML != 1) {
 		$lines[$i] =~ s/^\s*AddModule/# AddModule/;
  	    }

	    next;

  	}
  	elsif ($lines[$i] =~ m/^\#\s*AddModule/) {
	    
  	    # Extract module name using split
  	    my @mods = split(" ", $lines[$i]);
	    
  	    # Check if it exists in the XML
  	    my $checkXML = datastore::ApacheCheckModule($apachedoc,$mods[2]);
	    
  	    # If it does exist in XML, then uncomment line
  	    # If it does not exist in XML, leave it alone
	    
  	    if ($checkXML == 1) {
  		$lines[$i] =~ s/^#\s*AddModule/AddModule/;
  	    }

	    next;

  	}
	# Admin E-Mail
	if ($lines[$i] =~ m/^\s*#ServerAdmin/ || $lines[$i] =~ m/^\s*ServerAdmin/) {

	    my $adminemail = eval { $apachedoc->getElementsByTagName("ServerAdminEmail")->item(0)->getFirstChild->getData(); } || "";
	    $adminemail =~ s/ //g;

	    if ($adminemail ne "") {
		$lines[$i] = "ServerAdmin $adminemail\n";
	    }
	    else {
		$lines[$i] = "#ServerAdmin\n";
	    }
	}
	
	# Server Name
	elsif ($lines[$i] =~ m/^\s*#ServerName/ || $lines[$i] =~ m/^\s*ServerName/) {

	    my $servname = eval { $apachedoc->getElementsByTagName("ServerName")->item(0)->getFirstChild->getData(); } || "";
		
	    if ($servname ne "") {
		$lines[$i] = "ServerName $servname\n";
	    }
	    else {
		$lines[$i] = "#ServerName localhost\n";
	    }

	}
#  	# LockFile
#  	elsif ($lines[$i] =~ m/^\s*LockFile/) {
#  	    my $lockfile = eval { $apachedoc->getElementsByTagName("LockFile")->item(0)->getFirstChild->getData(); } || "";
#  	    if ($lockfile ne "") {
#  		$lines[$i] = "LockFile $lockfile\n";
#  	    }
#  	}
	
#  	# PidFile
#  	elsif ($lines[$i] =~ m/^\s*PidFile/) {
#  	    my $pidfile = eval { $apachedoc->getElementsByTagName("PidFile")->item(0)->getFirstChild->getData(); } || "";
#  	    if ($pidfile ne "") {
#  		$lines[$i] = "PidFile $pidfile\n";
#  	    }
#  	}

#  	# ScoreBoardFile
#  	elsif ($lines[$i] =~ m/^\s*ScoreBoardFile/) {
#  	    my $scoreboardfile = eval { $apachedoc->getElementsByTagName("ScoreBoardFile")->item(0)->getFirstChild->getData(); } || "";
#  	    $lines[$i] = "ScoreBoardFile $scoreboardfile\n";
#  	}
	
	# DocumentRoot
	elsif ($lines[$i] =~ m/^\s*DocumentRoot/) {
	    my $apachedocroot = eval { $apachedoc->getElementsByTagName("DocumentRoot")->item(0)->getFirstChild->getData(); } || "";
	    $lines[$i] = "DocumentRoot $apachedocroot\n";
	}

	# Minimum # of Spare Servers
	elsif ($lines[$i] =~ m/^\s*MinSpareServers/) {
	    my $minserv = eval { $apachedoc->getElementsByTagName("MinSpareServers")->item(0)->getFirstChild->getData(); } || "";
	    if ($minserv ne "") {
		$lines[$i] = "MinSpareServers $minserv\n";
	    }
	    else {
		$lines[$i] = "MinSpareServers 5\n";
	    }
	}
	# Maximum # of Spare Servers
	elsif ($lines[$i] =~ m/^\s*MaxSpareServers/) {
	    my $maxserv = eval { $apachedoc->getElementsByTagName("MaxSpareServers")->item(0)->getFirstChild->getData(); } || "";
	    if ($maxserv ne "") {
		$lines[$i] = "MaxSpareServers $maxserv\n";
	    }
	    else {
		$lines[$i] = "MaxSpareServers 20\n";
	    }
	}

	# Number of Start Servers
	elsif ($lines[$i] =~ m/^\s*StartServers/) {
	    my $strtserv = eval { $apachedoc->getElementsByTagName("StartServers")->item(0)->getFirstChild->getData(); } || "";
	    if ($strtserv ne "") {
		$lines[$i] = "StartServers $strtserv\n";
	    }
	    else {
		$lines[$i] = "StartServers 8\n";
	    }
	}

	# Maximum # of Clients
	elsif ($lines[$i] =~ m/^\s*MaxClients/) {
	    my $maxclients = eval { $apachedoc->getElementsByTagName("MaxClients")->item(0)->getFirstChild->getData(); } || "";
	    if ($maxclients ne "") {
		$lines[$i] = "MaxClients $maxclients\n";
	    }
	    else {
		$lines[$i] = "MaxClients 150\n";
	    }
	}

	# Maximum # of Requests per Child
	elsif ($lines[$i] =~ m/^\s*MaxRequestsPerChild/) {
	    my $maxrequests = eval { $apachedoc->getElementsByTagName("MaxRequestsPerChild")->item(0)->getFirstChild->getData(); } || "";
	    if ($maxrequests ne "") {
		$lines[$i] = "MaxRequestsPerChild $maxrequests\n";
	    }
	    else {
		$lines[$i] = "MaxRequestsPerChild 100\n";
	    }
	}
	
	# ServerSignature Off, On, or with Email
	elsif ($lines[$i] =~ m/^\s*ServerSignature/) {
	    my $servsig = eval { $apachedoc->getElementsByTagName("ServerSignature")->item(0)->getFirstChild->getData(); } || "";
	    if ($servsig ne "") {
		$lines[$i] = "ServerSignature $servsig\n";
	    }
	}

	# CacheNegotiatedDocuments on or off
	elsif ($lines[$i] =~ m/^#\s*CacheNegotiatedDocs\n/) {
	    my $cachedocs = $apachedoc->getElementsByTagName("CacheNegotiatedDocs")->item(0)->getFirstChild->getData();
	    if ($cachedocs eq "Yes") {
		$lines[$i] =~ s/^#\s*CacheNegotiatedDocs/CacheNegotiatedDocs/;
	    }
	}
	elsif ($lines[$i] =~ m/^\s*CacheNegotiatedDocs\n/) {
	    my $cachedocs = eval { $apachedoc->getElementsByTagName("CacheNegotiatedDocs")->item(0)->getFirstChild->getData(); } || "";
	    if ($cachedocs eq "No") {
		$lines[$i] =~ s/^\s*CacheNegotiatedDocs/# CacheNegotiatedDocs/;
	    }
	}

	# Use Canonical Name
	elsif ($lines[$i] =~ m/^\s*UseCanonicalName/) {
	    my $canonname = eval { $apachedoc->getElementsByTagName("UseCanonicalName")->item(0)->getFirstChild->getData(); } || "";
	    if ($canonname ne "") {
		$lines[$i] = "UseCanonicalName $canonname\n";
	    }
	}
	
	# HostLookUps on or off
	elsif ($lines[$i] =~ m/^\s*HostnameLookups/) {
	    my $hostnameup = eval { $apachedoc->getElementsByTagName("HostnameLookups")->item(0)->getFirstChild->getData(); } || "";
	    if ($hostnameup ne "") {
		$lines[$i] = "HostnameLookups $hostnameup\n";
	    }
	}

	# Index Options (fancy indexing or standard)
	elsif ($lines[$i] =~ m/^\s*IndexOptions/) {
	    my $indexoptions = eval { $apachedoc->getElementsByTagName("IndexOptions")->item(0)->getFirstChild->getData(); } || "";
	    $lines[$i] = "IndexOptions $indexoptions\n";
	}

	# Location of the TypesConfig file (mime.types)
	elsif ($lines[$i] =~ m/^\s*TypesConfig/) {
	    my $typesconf = eval { $apachedoc->getElementsByTagName("TypesConfig")->item(0)->getFirstChild->getData(); } || "";
	    if ($typesconf ne "") {
		$lines[$i] = "TypesConfig $typesconf\n";
	    }
	    else {
		$lines[$i] = "TypesConfig /etc/mime.types\n";
	    }
	}

	# Distinguish the type of default MIME type
	elsif ($lines[$i] =~ m/^\s*DefaultType/) {
	    my $defaulttype = eval { $apachedoc->getElementsByTagName("DefaultType")->item(0)->getFirstChild->getData(); } || "";
	    if ($defaulttype ne "") {
		$lines[$i] = "DefaultType $defaulttype\n";
	    }
	    else {
		$lines[$i] = "DefaultType text/plain\n";
	    }
	}

	# Location of the Error Log
	elsif ($lines[$i] =~m/^\s*ErrorLog/) {
	    my $errorlog = eval { $apachedoc->getElementsByTagName("ErrorLog")->item(0)->getFirstChild->getData(); } || "";
	    if ($errorlog ne "") {
		$lines[$i] = "ErrorLog $errorlog\n";
	    }
	}

	# Type of error message logged 
	elsif ($lines[$i] =~ m/^\s*LogLevel/) {
	    my $loglevel = eval { $apachedoc->getElementsByTagName("LogLevel")->item(0)->getFirstChild->getData(); } || "";
	    if ($loglevel ne "") {
		$lines[$i] = "LogLevel $loglevel\n";
	    }
	}

	# The length of the Timeout
	elsif ($lines[$i] =~ m/^\s*Timeout/) {
	    my $timeout = eval { $apachedoc->getElementsByTagName("Timeout")->item(0)->getFirstChild->getData(); } || "";
	    if ($timeout ne "") {
		$lines[$i] = "Timeout $timeout\n";
	    }
	}

	# Determine if Keep Alive is on or off
	elsif ($lines[$i] =~ m/^\s*KeepAlive /) {
	    my $keepalive = eval { $apachedoc->getElementsByTagName("KeepAlive")->item(0)->getFirstChild->getData(); } || "";
	    if ($keepalive ne "") {
		$lines[$i] = "KeepAlive $keepalive\n";
	    }
	}

	# Determine length of Max. Keep Alive requests
	elsif ($lines[$i] =~ m/^\s*MaxKeepAliveRequests/) {
	    my $maxkareq = eval { $apachedoc->getElementsByTagName("MaxKeepAliveRequests")->item(0)->getFirstChild->getData(); } || "";
	    if ($maxkareq ne "") {
		$lines[$i] = "MaxKeepAliveRequests $maxkareq\n";
	    }
	}
	
	# Determine length of Keep Alive timeouts
	elsif ($lines[$i] =~ m/^\s*KeepAliveTimeout/) {
	    my $katimeout = eval { $apachedoc->getElementsByTagName("KeepAliveTimeout")->item(0)->getFirstChild->getData(); } || "";
	    if ($katimeout ne "") {
		$lines[$i] = "KeepAliveTimeout $katimeout\n";
	    }
	}

	# Determine Language Priority
	elsif ($lines[$i] =~ m/^\s*LanguagePriority/) {
	    my $langprior = eval { $apachedoc->getElementsByTagName("LanguagePriority")->item(0)->getFirstChild->getData(); } || "";
	    if ($langprior ne "") {
		$lines[$i] = "LanguagePriority $langprior\n";
	    }
	}

	
	# IP/Port to listen on
#  	elsif ($lines[$i] =~ m/^\s*Listen/) {
#  	    my $listen = eval { $apachedoc->getElementsByTagName("Listen")->item(0)->getFirstChild->getData(); } || "";
#  	    if ($listen ne "") {
#  		$lines[$i] = "Listen $listen\n";
#  	    }
#  	}
	
#  	# Port to listen on
#  	elsif ($lines[$i] =~ m/^\s*Port/) {
#  	    my $port = eval { $apachedoc->getElementsByTagName("Port")->item(0)->getFirstChild->getData(); } || "80";
#  	    if ($port ne "") {
#  		$lines[$i] = "Port $port\n";
#  	    }
#  	}
	
	# Enable or unenable the SSL engine for the virtual host
	elsif ($lines[$i] =~ m/^\s*SSLEngine/) {

	    my $sslengine = eval { $apachedoc->getElementsByTagName("SSLEngine")->item(0)->getFirstChild->getData(); } || "";

	    print STDERR "Updating SSLEngine settings $sslengine\n";

	    if ($sslengine ne "") {
		if ($sslengine eq "Yes") {
		    $lines[$i] = "SSLEngine on\n";
		}
		else {
		    $lines[$i] = "SSLEngine off\n";
		}
	    }
	}

	#
	
    }

    # Set permissions for root web
    my $serveradmin = eval { $apachedoc->getElementsByTagName("ServerAdmin")->item(0)->getFirstChild->getData(); } || "";

    my %userInfo = datastore::userInfoByID($serveradmin);

    if ($userInfo{'loginname'} eq "") {
	$serveradmin = "root";
    }
    else {
	$serveradmin = $userInfo{'loginname'};
    }

    my $command = "/bin/chown -R $serveradmin:$serveradmin /var/www";
    print STDERR `/usr/bin/whoami`;
    print STDERR "Command = $command\n";
    my $result = `$command`;

    # Install the front page services if they don't exist
    if ((-s "/var/www/html/_vti_pvt/service.pwd") <= 0) {

	# Make sure fphttpd.conf exists
	if (open(FPFILE, ">/etc/httpd/conf/fphttpd.conf")) {

	    my $command = "/bin/cat $system{'httpd.conf'}";
	    my $result = `$command`;

	    print FPFILE "$result\n";

	    $command = "/bin/cat /etc/httpd/conf/vhosts.conf";
	    $result = `$command`;

	    print FPFILE "$result\n";

	    close(FPFILE);

	    $command = "/bin/chmod o+r /etc/httpd/conf/fphttpd.conf";
	    $result = `$command`;

	}

	`/bin/rm /usr/local/frontpage/we80.cnf`;

	$command = "echo \"0\n\n$serveradmin\n$serveradmin\n$serveradmin\ntemporary442\ntemporary442\n\" | /usr/local/frontpage/version4.0/bin/fpsrvadm_suid.exe -operation install -port 80 -servconf /etc/httpd/conf/fphttpd.conf";
	# WARNING:  Do not remove this STDERR!!
	print STDERR "Command for building front page = $command\n";
	$result = `$command`;

    }


    # Update the root web front page settings
    # Update the front page password
    my $result = `/bin/rm /var/www/html/_vti_pvt/service.pwd`;
    $result = `/bin/rm /var/www/html/_vti_pvt/service.grp`;

    # Update the password files
    # Rebuild the service.grp file
    if (open (FPFILE, ">/var/www/html/_vti_pvt/service.grp")) {
	print FPFILE "# -FrontPage-\n";
	print FPFILE "administrators: $serveradmin\n";
	
	print FPFILE "authors: ";
	
	close(FPFILE);
    }

    my $fpextensions = `/bin/cat $system{'datastore'}/apache.xml | /bin/grep \\\<FrontPageExtensions\\\>Yes`;

    # If front page is on, add it to the root web
    if ($fpextensions ne "") {

	# Make mock module
	`/bin/cp /usr/lib/apache/mod_example.so /usr/lib/apache/mod_frontpage.so`;
	`$system{'root'}/system/bin/chown root:root /usr/lib/apache/mod_example.so /usr/lib/apache/mod_frontpage.so`;
	`$system{'root'}/system/bin/chmod +x /usr/lib/apache/mod_frontpage.so`;
	
	# Rebuild the service.pwd file
	if (open (FPFILE, ">/var/www/html/_vti_pvt/service.pwd")) {
	    print FPFILE "# -FrontPage-\n";
	    close(FPFILE);
	}

    }
    else {

	# Make mock module
	`/bin/rm /usr/lib/apache/mod_frontpage.so`;

    }
    
    updateHTPasswd ($serveradmin, "/var/www/html/_vti_pvt/service.pwd");
    
    # Update .htaccess to play nicely with autoindex
    if (open(HTACCESS, "/var/www/html/.htaccess")) {
	
	my @htlines = <HTACCESS>;
	
	close(HTACCESS);
	
	if (open(HTACCESS, ">/var/www/html/.htaccess")) {
	    
	    foreach my $htline (@htlines) {
		
		if ($htline =~ m/IndexIgnore/) {
		    $htline = "<IfModule mod_autoindex.c>
  IndexIgnore .htaccess */.??* *~ *# */HEADER* */README* */_vti*
</IfModule>\n";
		}
		
		print HTACCESS $htline;
	    }
	    
	    close(HTACCESS);
	}
    }


    $command = "$system{'root'}/system/bin/chmod ug+ws /var/www/html/_vti_bin/shtml.exe";
    $result = `$command`;
    $command = "$system{'root'}/system/bin/chmod ug+ws /var/www/html/_vti_bin/fpcount.exe";
    $result = `$command`;
    $command = "$system{'root'}/system/bin/chmod ug+ws /var/www/html/_vti_bin/_vti_adm/admin.exe";
    $result = `$command`;
    $command = "$system{'root'}/system/bin/chmod ug+ws /var/www/html/_vti_bin/_vti_aut/author.exe";
    $result = `$command`;

    # Open config file
    if (open(FILE, ">$configFile")) {
    
	# Write lines
	foreach my $line (@lines) {
	    print FILE $line;
	}
	
	# Close config file
	close(FILE);

	print STDERR "Wrote httpd.conf back out\n";

    }
    else {
	print STDERR "WARNING:  Unable to write to httpd.conf\n";
    }

    # Clean-up
    $apachedoc->dispose();

    # Make sure fphttpd.conf exists
    if (open(FPFILE, ">/etc/httpd/conf/fphttpd.conf")) {
	
	my $command = "/bin/cat $system{'httpd.conf'}";
	my $result = `$command`;
	
	print FPFILE "$result\n";
	
	$command = "/bin/cat /etc/httpd/conf/vhosts.conf";
	$result = `$command`;

	print FPFILE "$result\n";
	
	close(FPFILE);
	
	$command = "/bin/chmod o+r /etc/httpd/conf/fphttpd.conf";
	$result = `$command`;
	
    }

    return(1);

}

# A sub-routine that stops the Apache web server
sub stopApache {

    print STDERR "Stopping apache with kill\n";

    # Stop Apache
    # Look at the run file
    if (open(FILE, "</var/run/httpd.pid")) {

	my $PID = <FILE>;
	close(FILE);

	my $command = "/bin/kill $PID";
	my $result = `$command`;

	close(FILE);

    }
    # Find the PID
    else {

	my $command = "/bin/ps axuw | /bin/grep /usr/sbin/httpd";
	my $result = `$command`;

	if ($result ne "") {

	    my @lines = split("\n", $result);
	    my @pieces = split(" ", $lines[0]);
	    
	    $command = "/bin/kill $pieces[1]";
	    $result = `$command`;

	}
	

    }

    return(1);
}

# A sub-routine that starts the Apache web server
sub startApache {

    # Gather up the module arguments
    my $command = '/bin/ls /usr/lib/apache/*.so | /bin/awk \'{\\
                                gsub(".*/","");\\
                                gsub("^mod_","");\\
                                gsub("^lib","");\\
                                gsub("\\.so$","");\\
				print toupper($0)}\'';
    my $result = `$command`;
    
    my @lines = split('\n', $result);
    
    $command = "/usr/sbin/httpd ";
    
    foreach my $module (@lines) {
	    
	# PUT BACK IN WHEN THROTTLE WORKS!!!
	if (!($module =~ m/throttle/) && !($module =~ m/auth_db/)) {
	    $command .= "-D HAVE_$module ";
	}
    }
    
    # Startup Apache
    $result = `$command`;

    print STDERR "Starting apache!! command = $command\n";

    return(1);

}

# A sub-routine that cycles the Apache web server
sub restartApache {

    my $command = "";
    my $result = "";

    # Reload Apache Configuration
    if (open(FILE, "</var/run/httpd.pid")) {

	my $PID = <FILE>;
	close(FILE);

	$command = "/bin/kill -s HUP $PID";

	print STDERR "Simple HUP restart\n";

	print STDERR `$command`;

    }

    # Check to make sure we are running 
    my $processes = `/bin/ps auwx | /bin/grep /usr/sbin/httpd | /usr/bin/wc -l`;
    $processes =~ s/\W//g;
    $processes = scalar($processes);

    print STDERR "Found $processes httpd daemons already running\n";

    if ($processes < 5) {

	# Remove any lock files
	my $command = "/bin/rm -f /var/lock/subsys/httpd /var/run/httpd.pid";
	my $result = `$command`;

	startApache();

	print STDERR "Apache manual restart!\n";

    }

    print STDERR "Completely finished with apache restart!\n";

    return(1);


}

# A sub-rotine to read the sendmail configuration, re-create
# the sendmail.cf configuration file, and cycle the server
sub updateSendmail {

    print STDERR "Updating sendmail\n";

    my %system = config::system();

    # Read the email configuration file
    my $parser = new XML::DOM::Parser;
    my $doc = $parser->parsefile("$system{'datastore'}/email.xml");

    my $masq = eval { $doc->getElementsByTagName("Masquerade")->item(0)->getFirstChild->getData(); } || "";

    $doc->dispose();

    # Re-build the sendmail.cf file - #1 Read in configuration file
    open (FILE, "</etc/mail/sendmail.mc");
    my @lines = <FILE>;
    my $numOfLines = @lines;
    close(FILE);

    my $haveGenericsTable = 0;

    for (my $i=0; $i<$numOfLines; $i++) {

	if ($lines[$i] =~ m/^dnl/) {
	    next;
	}
	elsif ($lines[$i] =~ m/^\s*FEATURE\(\`masquerade_envelope'\)/) {
	    $lines[$i] = "";
	}
	# Update Masquerade
	elsif ($lines[$i] =~ m/\s*MASQUERADE_AS\(.*\)/) {
	    $lines[$i] = "";
	}
	elsif ($lines[$i] =~ m/\s*FEATURE\(\`genericstable\'/) {
	    $haveGenericsTable = 1;
	}

    }

    # Add generics table to the mc file
    if ($haveGenericsTable == 0) {
	# Add generics domain feature
	push (@lines, "FEATURE(`genericstable', `hash -o /etc/mail/genericstable')dnl\n");
	push (@lines, "GENERICS_DOMAIN_FILE(`/etc/mail/genericsdomain')dnl\n");	
    }

    # Add envelope masquerading
    push(@lines,"FEATURE(`masquerade_envelope')dnl\n");
    push(@lines,"MASQUERADE_AS($masq)dnl\n");

    # Write the configuration file back out
    open(FILE, ">/etc/mail/sendmail.mc");

    foreach my $line (@lines) {
	if ($line eq "") {
	    next;
	}
	else {
	    print FILE $line;
	}
    }

    close(FILE);

    # Check for current sendmail.cf file
    my $result = "";
    if (-f "/etc/sendmail.cf") {
	$result = `/bin/cp -f /etc/sendmail.cf /etc/sendmail.cf.backup`;
    }

    # Re-make the .cf file
    $result = `/bin/chown root:root /etc/sendmail.cf`;
    $result = `/bin/chmod 647 /etc/sendmail.cf`;
    my $newcf = `/usr/bin/m4 /etc/mail/sendmail.mc`;

    if(open(CFFILE,">/etc/sendmail.cf")) {
	print CFFILE "$newcf";
	close(CFFILE);
    }
    
    $result = `/bin/chmod 644 /etc/sendmail.cf`;

    # Re-build the local-host-names file
    # Read the email configuration file
    my $parser2 = new XML::DOM::Parser;
    my $doc2 = $parser2->parsefile("$system{'datastore'}/virtualsites.xml");

    print STDERR "   virtualsites file parsed and ready to go\n";

    # Add masquerade to the local-host-names file
    if (open(FILE, ">/etc/mail/local-host-names")) {

	print STDERR "   **** Updating local-host-names\n";

	print FILE "$masq\n";

	foreach my $node ($doc2->getElementsByTagName("ServerName")) {

	    my $sitename = $node->getFirstChild->getData();
#	    print STDERR "   Adding localhostname $sitename\n";
	    print FILE "$sitename\n";

            # Translate off just the domain name for user@domain.com
            my @sections = split('\.', $sitename);
            my $numOfSections = scalar(@sections);
#	    print STDERR "NumOfSections = $numOfSections\n$sections[1] $sections[0]\n";
	    if ($numOfSections >= 2) {
		print FILE "$sections[$numOfSections-2].$sections[$numOfSections-1]\n";
	    }

	}

	close(FILE);

    }

    $doc2->dispose();

    # Re-start sendmail
    if (open(FILE, "</var/run/sendmail.pid")) {

	my @PIDlines = <FILE>;

	my $pid = $PIDlines[0];

	$result = `/bin/kill -s HUP $pid`;

	close (FILE);

    }
    else {
	print STDERR "WARNING:  Could not open \"/var/run/sendmail.pid\" for sendmail restart!\n";
    }

    # Check to make sure sendmail is running
    my $sendmailcheck = `/bin/ps awux | /bin/grep \"sendmail: accepting connections\"`;

    if ($sendmailcheck eq "") {
	`/usr/sbin/sendmail -bd`;
    }

    return(1);

}

# A sub-routine to update the genericstable file
sub updateSendmailGenericsTable {

    my %system = config::system();

    # Re-build the genericstable file for virtual hosting
    # Read the email configuration file
    my $parser = new XML::DOM::Parser;
    my $doc = $parser->parsefile("$system{'datastore'}/users.xml");

    # Add masquerade to the local-host-names file
    if (open(FILE, ">/etc/mail/genericstable")) {

	foreach my $node ($doc->getElementsByTagName("User")) {

	    my $loginname = eval { $node->getElementsByTagName("LoginName")->item(0)->getFirstChild->getData(); } || "";
	    my $email = eval { $node->getElementsByTagName("Email")->item(0)->getFirstChild->getData(); } || "";
	    $email =~ s/ //g;
	    $email =~ s/\n//g;

	    if ($email ne "") {
		print FILE "$loginname\t\t\t$email\n";
	    }

	}

	close(FILE);

    }

    $doc->dispose();

    # Rebuild hash map
    `/usr/bin/makemap hash $system{'root'}/system/reports/genericstable < /etc/mail/genericstable`;

    my @lines = ();
    if (open(NEWFILE, "$system{'root'}/system/reports/genericstable.db")) {
	@lines = <NEWFILE>;
	close(NEWFILE);
    }

    if (open(GENFILE, ">/etc/mail/genericstable.db")) {

	foreach my $line (@lines) {
	    print GENFILE "$line";
	}

	close(GENFILE);
    }

    return(1);

}

# A sub-routine to update the genericsdomain file
sub updateSendmailGenericsDomain {

    my %system = config::system();

    # Re-build the genericstable file for virtual hosting
    # Read the email configuration file
    my $parser = new XML::DOM::Parser;
    my $doc = $parser->parsefile("$system{'datastore'}/virtualsites.xml");

    # Add masquerade to the local-host-names file
    if (open(FILE, ">/etc/mail/genericsdomain")) {

	my $fqdn = `/bin/hostname -f`;

	print FILE "$fqdn";

	foreach my $node ($doc->getElementsByTagName("ServerName")) {

	    my $sitename = eval { $node->getFirstChild->getData(); } || "";

            # Translate off just the domain name for user@domain.com
            my @sections = split('\.', $sitename);
            my $numOfSections = scalar(@sections);
	    if ($numOfSections >= 2) {
		print FILE "$sections[$numOfSections-2].$sections[$numOfSections-1]\n";
	    }

	    if ($sitename ne "") {
		print FILE "$sitename\n";
	    }

	}

	close(FILE);

    }

    $doc->dispose();

    return(1);

}


# A sub-routine to rebuild the email aliases file
sub updateEmailAliases {

    my $configFile = "/etc/aliases";

    my %system = config::system();

    # Read in the current aliases file
    my @lines = ();
    if (open(FILE, "<$configFile")) {
	@lines = <FILE>;
	close(FILE);
    }

    # Read the aliases
    my $parser = new XML::DOM::Parser;
    my $doc = $parser->parsefile("$system{'datastore'}/email.xml");

    # Run through each alias and either add or update the corresponding line
    foreach my $alias ($doc->getElementsByTagName("Alias")) {

	my $name = $alias->getAttribute("name");

	# Calculate member string
	my $members = "";
	foreach my $member ($alias->getElementsByTagName("Member")) {
	    $members .= $member->getFirstChild->getData() . ", ";
	}
	
	# Get rid of the last comma
	$members = substr($members, 0, length($members)-2);

	my $numOfLines = @lines;

	# Run through each line of the file to find alias
	my $foundAlias = 0;
	for (my $i=0; $i<$numOfLines; $i++) {

	    if ($lines[$i] =~ m/#/) {
		next;
	    }
	    # If we found the alias, then update it
	    elsif ($lines[$i] =~ m/^$name:/) {

		$foundAlias = 1;
		$lines[$i] = "$name: $members\n";
	    }
	}

	# Check to see if we need to add this alias
	if ($foundAlias == 0) {
	    push (@lines, "$name: $members\n");
	}
    }

    # Write the file back out
    if (open(FILE, ">$configFile")) {

	foreach my $line (@lines) {
	    print FILE "$line";
	}

	close(FILE);

    }

    # Rebuild aliases
    my $command = "/usr/bin/makemap hash $system{'root'}/system/aliases.db < /etc/aliases";
    my $result = `$command`;

    my @lines = ();
    if (open(ALIASFILE,"<$system{'root'}/system/aliases.db")) {

	binmode(ALIASFILE);

	my $buffer = "";

	# Write out the new aliases file (Binary file)
	if (open(NEWALIAS,">/etc/aliases.db")) {

	    binmode(NEWALIAS);

	    my $bytesread = 0;
	    
	    while ($bytesread = read(ALIASFILE, $buffer, 1024)) {

#		print STDERR "  - Read $bytesread bytes from aliases.db\n";
		syswrite(NEWALIAS, $buffer, $bytesread);
	    }
	    
	    close(NEWALIAS);
	}

	close(ALIASFILE);
	    
    }


}

# A sub-routine to delete an alias from the email aliases file
sub delEmailAlias {

    my $aliasName = shift || return;

    print STDERR "About to delete an alias of $aliasName\n";

    my %system = config::system();

    # Read the aliases and update the XML
    my $parser = new XML::DOM::Parser;
    my $doc = $parser->parsefile("$system{'datastore'}/email.xml");

    # Run through each alias and either add or update the corresponding line
    foreach my $alias ($doc->getElementsByTagName("Alias")) {

	my $name = $alias->getAttribute("name");

	print STDERR "Found $name\n";

	# Remove this alias
	if ($name eq $aliasName) {
	    print STDERR "Removing child\n";
	    $doc->getFirstChild->removeChild($alias);
	}

    }
    
    $doc->printToFile("$system{'datastore'}/email.xml");

    # Delete the alias from the aliases file
    my $configFile = "/etc/aliases";

    # Read in the current aliases file
    my @lines = ();
    if (open(FILE, "<$configFile")) {
	@lines = <FILE>;
	close(FILE);
    }

    # Write the file back out
    if (open(FILE, ">$configFile")) {

	foreach my $line (@lines) {

	    # If this is not the alias we are getting rid of
	    if (!($line =~ m/^$aliasName:/)) {
		print FILE "$line";
	    }

	}

	close(FILE);

    }

    my $command = "/usr/bin/makemap hash $system{'root'}/system/aliases.db < /etc/aliases";
    my $result = `$command`;

    my @lines = ();
    if (open(ALIASFILE,"<$system{'root'}/system/aliases.db")) {

	binmode(ALIASFILE);

	my $buffer = "";

	# Write out the new aliases file (Binary file)
	if (open(NEWALIAS,">/etc/aliases.db")) {

	    binmode(NEWALIAS);

	    my $bytesread = 0;
	    
	    while ($bytesread = read(ALIASFILE, $buffer, 1024)) {

#		print STDERR "  - Read $bytesread bytes from aliases.db\n";
		syswrite(NEWALIAS, $buffer, $bytesread);
	    }
	    
	    close(NEWALIAS);
	}

	close(ALIASFILE);
	    
    }

}

# A sub-routine to update the password in a .htpasswd file
# This sub-routine pulls information from the existing shadow
# password file and populates/adds the .htpasswd file
sub updateHTPasswd {

    my $user = shift || return();
    my $file = shift || return();

    # Open the shadow password file
    my $password = "";
    if (open(FILE, "/etc/shadow")) {

	# Find the correct user
	my @lines = <FILE>;

	foreach my $line (@lines) {
	    if ($line =~ m/^$user:([^:]*):/) {
		$password = $1;
	    }
	}

	close (FILE);

    }

    # Find the user
    if (open(FILE, "<$file")) {

	my $foundUser = 0;
	my @lines = <FILE>;

	foreach my $line (@lines) {

	    if ($line =~ m/$user:/) {
		$foundUser = 1;
		$line = "$user:$password\n";
	    }
	}

	if ($foundUser == 0) {
	    push (@lines, "$user:$password\n");
	}

	# Write the file
	if (open(FILE, ">$file")) {

	    foreach my $line (@lines) {
		print FILE "$line";
	    }

	    close (FILE);
	}
	
    }
    
}

# A sub-routine to remove a user from a .htpasswd file
sub delHTPasswd {

    my $user = shift || return();
    my $file = shift || return();

    # Find the user
    if (open(FILE, "<$file")) {

	my $foundUser = 0;
	my @lines = <FILE>;

	foreach my $line (@lines) {

	    if ($line =~ m/$user:/) {
		$foundUser = 1;
	    }
	}

	if ($foundUser) {

	    # Write the file
	    if (open(FILE, ">$file")) {
		
		foreach my $line (@lines) {

		    # Print every line but the delete user
		    if (!($line =~ m/^$user:/)) {
			print FILE "$line";
		    }
		}
		
		close (FILE);
	    }
	}
	
    }
    
}

# A routine to handle system backups
sub systemBackup {

    my %system = config::system;

    my $type = shift || return();

    my %items = {};

    foreach my $parameter (@_) {
	$items{$parameter} = 1;
    }

    # Build a random file suffix
    srand();
    my $suffix = int(rand(5000));
    my $date = sprintf ("%d%d%d", localtime->year()+1900, localtime->mon(), localtime->mday());

    my $file = "Cpqcfgbackup_$suffix" . "_" . "$date.tgz";

    # Build file list by groups
    my $network = "/etc/ftpusers /etc/pop3users /etc/telnetusers /etc/sshusers /etc/cpqcfgusers /etc/sysconfig/network-scripts/ifcfg-eth0 /etc/sysconfig/network-scripts/ifcfg-eth1 /etc/sysconfig/network";
    my $home = "/home/*";
    my $users = "/etc/passwd /etc/shadow /var/cpqcfg/cgi-bin/CompaqConfigurator/siteAdmin/.htpasswd /var/cpqcfg/cgi-bin/CompaqConfigurator/systemAdmin/.htpasswd /var/cpqcfg/cgi-bin/CompaqConfigurator/user/.htpasswd";
    my $groups = "/etc/group /etc/gshadow";
    my $primarysite = "/var/www/* /usr/local/frontpage/*.cnf /etc/httpd/conf/ssl.* /etc/httpd/conf/httpd.conf";
    my $virtualsites = "/home/VirtualSites/* /usr/local/frontpage/*.cnf /etc/httpd/conf/vhosts.conf /etc/httpd/conf/fphttpd.conf /etc/logrotate.d/*";
    my $sendmail = "/etc/mail/* /etc/aliases /var/spool/mail/* /var/spool/mqueue/*";
    my $majordomo = "/etc/majordomo.cf /var/lib/majordomo/*";
    my $datastore = "$system{'datastore'}/*";
    my $reports = "$system{'root'}/system/reports/*";

    # We start with files that are ALWAYS included
    my $filelist = "$users $groups $datastore $reports $network";

    # Build the correct backup file
    if ($items{'syscfg'} == 1) {
	$filelist .= "";
    }
    if ($items{'primary'} == 1) {
	$filelist .= " $primarysite";
    }
    if ($items{'virtual'} == 1) {
	$filelist .= " $virtualsites";
    }
    if ($items{'user'} == 1) {
	$filelist .= " $home";
    }
    if ($items{'mail'} == 1) {
	$filelist .= " $sendmail";
    }

    my $command = "$system{'root'}/system/bin/tar -P --ignore-failed-read --preserve-order --preserve-permissions -c $filelist -O | /bin/gzip -c |";

    print STDERR "Command = $command\n";

    # Do the backup
    if (open(BACKUP, "$command")) {
	
	# Make the backup using controled data release
	while (!eof(BACKUP)) {
	    
	    my $buffer = "";
	    read (BACKUP, $buffer, $system{'backupbuffersize'});
	    print $buffer;
	    
	}
	
	close(BACKUP);
	
    }

}

# A sub-routine to update log configurations
sub updateLogConfig {

    print STDERR "Calling updateLogConfig in custom\n";

    my %system = config::system();

    # Step 1:  Parse information about log rotation from XML
    my $parser = new XML::DOM::Parser;
    my $doc = $parser->parsefile("$system{'datastore'}/general.xml");

    my $date = `/bin/date`;

    # Grab the values for the log files
    my $usesyslogdserver = eval { $doc->getElementsByTagName("Logs")->item(0)->getElementsByTagName("UseSyslogdServer")->item(0)->getFirstChild->getData(); } || "";
    my $syslogdserver = eval { $doc->getElementsByTagName("Logs")->item(0)->getElementsByTagName("SyslogdServer")->item(0)->getFirstChild->getData(); } || "";
    my $syslog = eval { $doc->getElementsByTagName("Logs")->item(0)->getElementsByTagName("Syslog")->item(0)->getFirstChild->getData(); } || "";
    my $weblog = eval { $doc->getElementsByTagName("Logs")->item(0)->getElementsByTagName("Weblog")->item(0)->getFirstChild->getData(); } || "";
    my $maillog = eval { $doc->getElementsByTagName("Logs")->item(0)->getElementsByTagName("Maillog")->item(0)->getFirstChild->getData(); } || "";

    my $weblogfile = "
/var/log/httpd/access_log {
    missingok
    postrotate
        /bin/kill -HUP `cat /var/run/httpd.pid 2>/dev/null` 2> /dev/null || true
    endscript
}

/var/log/httpd/agent_log {
    missingok
    postrotate
        /bin/kill -HUP `cat /var/run/httpd.pid 2>/dev/null` 2> /dev/null || true
    endscript
}

/var/log/httpd/error_log {
    missingok
    postrotate
        /bin/kill -HUP `cat /var/run/httpd.pid 2>/dev/null` 2> /dev/null || true
    endscript
}

/var/log/httpd/referer_log {
    missingok
    postrotate
        /bin/kill -HUP `cat /var/run/httpd.pid 2>/dev/null` 2> /dev/null || true
    endscript
}
";

    my $syslogfile = "/var/log/messages {
    postrotate
/bin/kill -HUP `cat /var/run/syslogd.pid 2> /dev/null` 2> /dev/null || true
    endscript
}

/var/log/secure {
    postrotate
/bin/kill -HUP `cat /var/run/syslogd.pid 2> /dev/null` 2> /dev/null || true
    endscript
}

/var/log/maillog {
/bin/kill -HUP `cat /var/run/syslogd.pid 2> /dev/null` 2> /dev/null || true
    endscript
}

/var/log/spooler {
    postrotate
/bin/kill -HUP `cat /var/run/syslogd.pid 2> /dev/null` 2> /dev/null || true
    endscript
}

/var/log/boot.log {
    postrotate
/bin/kill -HUP `cat /var/run/syslogd.pid 2> /dev/null` 2> /dev/null || true
    endscript
}

/var/log/cron {
    postrotate
/bin/kill -HUP `cat /var/run/syslogd.pid 2>/dev/null` 2>/dev/null || true
    endscript
}
";

    # weblog
    if ($weblog eq "Once a Month") {
	$weblogfile =~ s/missingok/missingok\n     monthly/g;
    }
    elsif ($weblog eq "When greater than 5 MB") {
	$weblogfile =~ s/missingok/missingok\n     size=5M/g;
    }
    elsif ($weblog eq "When greater than 10 MB") {
	$weblogfile =~ s/missingok/missingok\n     size=10M/g;
    }
    elsif ($weblog eq "When greater than 50 MB") {
	$weblogfile =~ s/missingok/missingok\n     size=50M/g;
    }
    else {
	$weblogfile =~ s/missingok/missingok\n     weekly/g;
    }

    # syslog
    if ($syslog eq "Once a Month") {
	$syslogfile =~ s/postrotate/monthly\n     postrotate/g;
    }
    elsif ($syslog eq "When greater than 5 MB") {
	$syslogfile =~ s/postrotate/size=5M\n     postrotate/g;
    }
    elsif ($syslog eq "When greater than 10 MB") {
	$syslogfile =~ s/postrotate/size=10M\n     postrotate/g;
    }
    elsif ($syslog eq "When greater than 50 MB") {
	$syslogfile =~ s/postrotate/size=50M\n     postrotate/g;
    }
    else {
	$syslogfile =~ s/postrotate/weekly\n     postrotate/g;
    }

    # maillog
    if ($maillog eq "Once a Month") {
	$syslogfile =~ s/\/var\/log\/maillog {/\/var\/log\/maillog {\n    postrotate\n    monthly\n/;
    }
    elsif ($maillog eq "When greater than 5 MB") {
	$syslogfile =~ s/\/var\/log\/maillog {/\/var\/log\/maillog {\n    postrotate\n    size=5M\n/;
    }
    elsif ($maillog eq "When greater than 10 MB") {
	$syslogfile =~ s/\/var\/log\/maillog {/\/var\/log\/maillog {\n    postrotate\n    size=10M\n/;
    }
    elsif ($maillog eq "When greater than 50 MB") {
	$syslogfile =~ s/\/var\/log\/maillog {/\/var\/log\/maillog {\n    postrotate\n    size=50M\n/;
    }
    else {
	$syslogfile =~ s/\/var\/log\/maillog {/\/var\/log\/maillog {\n    postrotate\n    weekly\n/;
    }

    # Step 2:  Build new log rotation configuration file
    my $newlogconfig = "
# Modified by Compaq Configurator - $date

# see \"man logrotate\" for details
# rotate log files weekly
daily

# keep 4 weeks worth of backlogs
rotate 4

# send errors to root
errors root

# create new (empty) log files after rotating old ones
create

# uncomment this if you want your log files compressed
compress

# RPM packages drop log rotation information into this directory
include /etc/logrotate.d

# no packages own lastlog or wtmp -- we'll rotate them here
/var/log/wtmp {
    daily
    create 0664 root utmp
    rotate 1
}
";

    # Step 2a:  Write syslog file out
    if (open(FILE, ">/etc/logrotate.d/syslog")) {

	print FILE $syslogfile;

	close (FILE);

    }

    # Step 2b:  Write apache file out
    if (open(FILE, ">/etc/logrotate.d/apache")) {

	print FILE $weblogfile;

	close (FILE);

    }

    # Step 3:  Write the new log configuration file out
    if (open(FILE, ">/etc/logrotate.conf")) {

	print FILE $newlogconfig;

	close (FILE);

    }

    # Step 4:  Open the syslog.conf file
    if (open (FILE, "</etc/syslog.conf")) {

	print STDERR "Opened syslog.conf read only mode  usesyslogdserver=$usesyslogdserver\n";

	# Step 5:  Setup a syslogd server
	my @lines = <FILE>;

	close (FILE);

	# Step 6:  Write the file back out
	if (open (FILE, ">/etc/syslog.conf")) {

	    foreach my $line (@lines) {

		# Modify the syslog line to log to a server if requested
		if ($line =~ m/^\*\.info;/ && $usesyslogdserver eq "Yes") {
		    print STDERR "Matched info line and changing\n";
		    $line = "*.info;mail.none;authpriv.none                          \@$syslogdserver\n";
		}
		elsif ($line =~ m/^\*\.info;/) {
		    $line = "*.info;mail.none;authpriv.none                          /var/log/messages\n";
		}

		print FILE "$line";

	    }

	    close(FILE);

	}

    }

    print STDERR "Trying to open cpqcfg log rotation config in write mode\n";

    # Re-build the cpqcfg log rotation config. file
    if(open(LOGFILE, ">/etc/logrotate.d/cpqcfg")) {

	print STDERR "Writing out cpqcfg log rotation config file\n";

	print LOGFILE "
# cpqcfg log rotation
/var/cpqcfg/httpd/log/access_log {
     missingok
     size=5M
     postrotate
	/etc/rc.d/init.d/cpqcfg stop 2> /dev/null 1> /dev/null
	/etc/rc.d/init.d/cpqcfg start 2> /dev/null 1> /dev/null
     endscript
}

/var/cpqcfg/httpd/log/error_log {
     missingok
     size=5M
     postrotate
	/etc/rc.d/init.d/cpqcfg stop 2> /dev/null 1> /dev/null
	/etc/rc.d/init.d/cpqcfg start 2> /dev/null 1> /dev/null
     endscript
}
";

	close(LOGFILE);

    }

}

# A sub-routine to update the SNMP settings for the system
sub updateSNMPConfig {

    my %system = config::system();

    # Step 1:  Parse information about SNMP settings
    my $parser = new XML::DOM::Parser;
    my $doc = $parser->parsefile("$system{'datastore'}/network.xml");

    # Grab the values for the log files
    my $rocomm = eval { $doc->getElementsByTagName("SNMP")->item(2)->getElementsByTagName("ROCommunity")->item(0)->getFirstChild->getData(); } || "";
    my $rwcomm = eval { $doc->getElementsByTagName("SNMP")->item(2)->getElementsByTagName("RWCommunity")->item(0)->getFirstChild->getData(); } || "";
    my $trapcomm = eval { $doc->getElementsByTagName("SNMP")->item(2)->getElementsByTagName("TrapCommunityName")->item(0)->getFirstChild->getData(); } || "";
    my $trapaddr = eval { $doc->getElementsByTagName("SNMP")->item(2)->getElementsByTagName("TrapServerAddress")->item(0)->getFirstChild->getData(); } || "";

    # Step 2: Update the snmp.conf file
    if (open (FILE, "</usr/share/snmp/snmpd.conf")) {

	print STDERR "SNMP Update - Read in the snmpd.conf file\n";

	my @lines = <FILE>;

	close (FILE);

	if (open (FILE, ">/usr/share/snmp/snmpd.conf")) {

	    my $addrocomm = 0;
	    my $addrwcomm = 0;
	    my $addtrapcomm = 0;
	    my $addtrapaddr = 0;

	    foreach my $line (@lines) {

		print STDERR "$line";

		# Find the RO community string, but avoid the Compaq setup
		if ($line =~ m/rwcommunity.*localcomm/) {
		    print FILE "$line";
		}
		elsif ($line =~ m/^rocommunity/) {
		    if ($rocomm ne "") {
			$line = "rocommunity $rocomm\n";
			print FILE "$line";
			$addrocomm = 1;
		    }
		}
		elsif ($line =~ m/^rwcommunity/) {
		    if ($rwcomm ne "") {
			$line = "rwcommunity $rwcomm\n";
			print FILE "$line";
			$addrwcomm = 1;
		    }
		}
		# Update the trap server
		elsif (($line =~ m/^trapsink/) && !($line =~ m/127.0.0.1/)) {
		    if ($trapaddr ne "") {
			$line = "trapsink $trapaddr\n";
			print FILE "$line";
			$addtrapaddr = 1;
		    }
		}
		elsif ($line =~ m/^trapcommunity/) {
		    if ($trapcomm ne "") {
			$line = "trapcommunity $trapcomm\n";
			print FILE "$line";
			$addtrapcomm = 1;
		    }
		}
		else {
		    print FILE "$line";
		}
	    }

	    # Add community lines if they don't exist
	    if ($addrocomm != 1 && $rocomm ne "") {
		print FILE "rocommunity $rocomm\n";
	    }
	    if ($addrwcomm != 1 && $rwcomm ne "") {
		print FILE "rwcommunity $rwcomm\n";
	    }
	    if ($addtrapcomm != 1 && $trapcomm ne "") {
		print FILE "trapcommunity $trapcomm\n";
	    }
	    if ($addtrapaddr != 1 && $trapaddr ne "") {
		print FILE "trapsink $trapaddr\n";
	    }

	    close (FILE);

	    print STDERR "Wrote snmpd.conf file out.  [$rocomm, $rwcomm, $trapcomm, $trapaddr] ($addrocomm, $addrwcomm, $addtrapcomm, $addtrapaddr)\n";
	}
    }

    # Cycle snmp daemon
    my $command = "/usr/bin/killall -HUP snmpd";
    my $result = `$command`;

}

# Get the date/time of the SSL files
#  Return key~csr~crt
sub getApacheSSLdates {

    my $result = `/bin/ls -l --full-time /etc/httpd/conf/ssl.key/server.key`;
    my @values = split(' ', $result);
    my $return = "$values[5] $values[6] $values[7] $values[8] $values[9] ";

    $result = `/bin/ls -l --full-time /etc/httpd/conf/ssl.csr/server.csr`;
    @values = split(' ', $result);
    $return .= "~$values[5] $values[6] $values[7] $values[8] $values[9] ";

    $result = `/bin/ls -l --full-time /etc/httpd/conf/ssl.crt/server.crt`;
    @values = split(' ', $result);
    $return .= "~$values[5] $values[6] $values[7] $values[8] $values[9] ";

    return ($return);

}

# Grab the current Apache SSL Key
sub getApacheSSLKey {

    my $sslkey = "";

    if (open (SSLFILE, "</etc/httpd/conf/ssl.key/server.key")) {

	my @sslkeylines = <SSLFILE>;

	foreach my $line (@sslkeylines) {
	    $sslkey .= "$line";
	}

	close (SSLFILE);
    }
    else {
	print STDERR "Could not open server.key\n";
    }

    return ($sslkey);

}

# Grab the current Apache CSR
sub getApacheSSLCSR {

    my $sslkey = "";

    if (open (SSLFILE, "</etc/httpd/conf/ssl.csr/server.csr")) {

	my @sslkeylines = <SSLFILE>;

	foreach my $line (@sslkeylines) {
	    $sslkey .= "$line";
	}

	close (SSLFILE);
    }
    else {
	print STDERR "Could not open server.csr\n";
    }

    return ($sslkey);

}

# Grab the current Apache CRT
sub getApacheSSLCRT {

    my $sslkey = "";

    if (open (SSLFILE, "</etc/httpd/conf/ssl.crt/server.crt")) {

	my @sslkeylines = <SSLFILE>;

	foreach my $line (@sslkeylines) {
	    $sslkey .= "$line";
	}

	close (SSLFILE);
    }
    else {
	print STDERR "Could not open server.crt\n";
    }

    return ($sslkey);

}

# Update the current key files
sub updateApacheSSLKeys {

    my $sslkey = shift || return();
    my $sslcsr = shift || return();
    my $sslcrt = shift || return();
    
    print STDERR "About to update SSL files\n";
    
    # Update server key
    if (open (SSLFILE, ">/etc/httpd/conf/ssl.key/server.key")) {
	
	print SSLFILE "$sslkey";
	
	close (SSLFILE);

	print STDERR "Wrote out server key\n";
    }
    else {
	print STDERR "Could not open server.key for update\n";
    }
    
    # Update Certificate Request
    if (open (SSLFILE, ">/etc/httpd/conf/ssl.csr/server.csr")) {
	
	print SSLFILE "$sslcsr";
	
	close (SSLFILE);
	
       print STDERR "Wrote out certificate request\n";
    }
    else {
	print STDERR "Could not open server.csr for update\n";
    }
    
    # Update Certificate
    if (open (SSLFILE, ">/etc/httpd/conf/ssl.crt/server.crt")) {
	
	print SSLFILE "$sslcrt";
	
	close (SSLFILE);
	
	print STDERR "Wrote out certificate\n";
    }
    else {
	print STDERR "Could not open server.crt for update\n";
    }

    return (1);

}


# A sub-routine to re-build the server key, csr, and crt
sub apacheSSLKeyRegen {

    my $passphrase = shift || "";
    my $country = shift || "";
    my $state = shift || "";
    my $city = shift || "";
    my $orgname = shift || "";
    my $orgunit = shift || "";
    my $cname = shift || "";
    my $email = shift || "";
    my $challenge = shift || "";
    my $coname = shift || "";

    my %system = config::system();

    my $keyresponse = "$passphrase\n$passphrase\n";
    my $csrresponse = "$passphrase\n$country\n$state\n$city\n$orgname\n$orgunit\n$cname\n$email\n$challenge\n$coname\n";
    my $crtresponse = "$passphrase\n$country\n$state\n$city\n$orgname\n$orgunit\n$cname\n$email\n";

    # Step 0:  Read in some randomness
    my $rand = "";
    if (open (RANDFILE, "/dev/urandom")) {

	read (RANDFILE, $rand, 4096);

	close (RANDFILE);
    }

    `/bin/echo \"$rand\" > /tmp/rand.txt`;

    # Step 1:  Build new key
    my $command = "/bin/echo \"$keyresponse\" | /usr/bin/openssl genrsa -des3 -rand /tmp/rand.txt 1024 2> /dev/null";
    my $sslkey = `$command`;

    if (open(FILE, ">/etc/httpd/conf/ssl.key/server.key")) {
	print FILE "$sslkey";
	close (FILE);
    }

    `/bin/echo "$sslkey" > $system{'root'}/system/server.key`;

    print STDERR "Finished key\n";

    # Step 2:  Build new certificate request
    $command = "/bin/echo \"$csrresponse\" | /usr/bin/openssl req -new -key $system{'root'}/system/server.key 2> /dev/null";
    my $sslcsr = `$command`;

    if (open(FILE, ">/etc/httpd/conf/ssl.csr/server.csr")) {
	print FILE "$sslcsr";
	close (FILE);
    }

    print STDERR "Finished certificate request\n";


      # Step 3:  Remove the pass phrase (It makes the httpd server pause for a pass phrase)
      $command = "echo \"$passphrase\n\" | /usr/bin/openssl rsa -in $system{'root'}/system/server.key 2> /dev/null";
      my $sslcrt_nopass = `$command`;

      if (open(FILE, ">/etc/httpd/conf/ssl.key/server.key")) {

  	print FILE "$sslcrt_nopass";

  	close (FILE);
      }

    `/bin/echo \"$sslcrt_nopass\" > $system{'root'}/system/server.key`;

    print STDERR "Finished with removal of passphrase!\n";

    # Step 4:  Build temporary certificate
    $command = "/bin/echo \"$crtresponse\" | /usr/bin/openssl req -new -key $system{'root'}/system/server.key -x509 -days 365 2> /dev/null";
    my $sslcrt = `$command`;

    if (open(SSLFILE, ">/etc/httpd/conf/ssl.crt/server.crt")) {

	print SSLFILE "$sslcrt";

	close (SSLFILE);

    }
    else {
	print STDERR "WARNING:  DID NOT WRITE CRT TO FILE!!!\n";
    }

    print STDERR "Finished certificate\n";

    # Clean-up
    `/bin/rm $system{'root'}/system/server.key`;
    `/bin/rm $system{'root'}/system/server.crt`;
    `/bin/rm /tmp/rand.txt`;

    # Step 5:  Hard-Cycle server
    $command = `/usr/bin/killall /usr/sbin/httpd`;
    restartApache();

}

# Update the SSH settings
sub updateSSH {

    my %system = config::system();

    my $regenkey = shift || "";

    if ($regenkey eq "Yes") {
	my $command = "/bin/rm /etc/ssh/ssh_host_key";
	my $result = `$command`;
	$command = "/bin/rm /etc/ssh/ssh_host_key.pub";
	$result = `$command`;
	$command = "/usr/bin/ssh-keygen -q -f /etc/ssh/ssh_host_key";
	$result = `$command`;
#	print STDERR "SSH Key Rebuild Command = $command\n";
#	print STDERR "Result = $result\n";
    }

    # Step 1:  Grab SSH settings
    my $query = 'WHERE
           <SSHSetup>
             <RootLogin>$rootlogin</RootLogin>
             <KeyRegenInterval>$interval</KeyRegenInterval>
           </SSHSetup>
        IN "file:' . $system{'datastore'} . '/network.xml"
        CONSTRUCT $rootlogin<TD>$interval
        ';

    my $result = XML::QL->query($query);

    my @values = split('<TD>', $result);

    my $rootlogin = $values[0];
    my $interval = $values[1];
    my $sslkey = "";

    print STDERR "rootlogin = $rootlogin  &   interval = $interval\n";

    # Step 2:  Generate new key if not given
    # Automagically regenerate key
    if ($sslkey eq "") {
    }
    # Populate with given key
    else {
    }

    # Step 3:  Update sshd_config file
    if (open(FILE, "</etc/ssh/sshd_config")) {

	my @lines = <FILE>;

	close(FILE);

	if (open (FILE, ">/etc/ssh/sshd_config")) {

	    foreach my $line (@lines) {

		# Change permit root login
		if ($line =~ m/^PermitRootLogin/) {
		    if ($rootlogin eq "Yes") {
			$line = "PermitRootLogin yes\n";
		    }
		    else {
			$line = "PermitRootLogin no\n";
		    }
		}

		# Change update interval
		if ($line =~ m/^KeyRegenerationInterval/ && $interval ne "") {
		    $line = "KeyRegenerationInterval $interval\n";
		}

		print FILE "$line";
	    }

	    close (FILE);
	}
    }

    # Step 4;  Cycle sshd
    my $command = "";
    if (open(FILE, "</var/run/sshd.pid")) {

	my $PID = <FILE>;
	close(FILE);

	$command = "/bin/kill -s HUP $PID";

	print STDERR "Simple HUP restart\n";

    }

    # Check to make sure we are running
    my $processes = `/bin/ps auwx | /bin/grep sshd`;

    if (!($processes =~ m/\/usr\/sbin\/sshd/)) {

	print STDERR "Restarting SSHD the hard way!\n";

	# Stop the server
	$command = "/usr/bin/killall /usr/sbin/sshd";
	my $result = `$command`;

	# Remove any lock files
	$command = "/bin/rm -f /var/lock/subsys/sshd /var/run/sshd.pid";
	$result = `$command`;

	$command = "/usr/sbin/sshd";

    }

    # Cycle network services
    updateServices();

    print STDERR "Starting SSHD!! command = $command\n";

    print STDERR `$command`;

}

# A sub-routine to upgrade the system
sub upgradeSystem {

    my $force = shift || "";

    my %system = config::system();

    # Step 1: Backup the datastore and webalizer data
    my $result = `/bin/mkdir $system{'datastore'}/backup`;
    print STDERR "$result\n";
    $result = `/bin/cp $system{'datastore'}/* $system{'datastore'}/backup`;
    print STDERR "$result\n";
    $result = `/bin/mkdir /var/cpqcfg/html/webalizer/backup`;
    print STDERR "$result\n";
    $result = `/bin/cp /var/cpqcfg/html/webalizer/* /var/cpqcfg/html/webalizer/backup`;
    print STDERR "$result\n";

    # Step 2: Apply the upgrade
    my $errorcode = "";
    if ($force eq "Yes") {
	$errorcode = system("/bin/rpm --replacefiles --force -U $system{'root'}/util/upgrade.rpm");
    }
    else {
	$errorcode = system("/bin/rpm -U $system{'root'}/util/upgrade.rpm");
    }

    print "Errorcode = $errorcode\n";

    # Step 3: Restore the datastore
    $result = `/bin/cp $system{'datastore'}/backup/* $system{'datastore'}/`;
    print STDERR "$result\n";

    # Cycle apache
    my $command = "$system{'root'}/system/custom/custom update_apache";
    $result = `$command`;

    return(1);

}

# A sub-routine to install network port "booby traps"
sub updateNetPortAlert {

    print STDERR "Made it to update net port alert\n";

    my %system = config::system();

    # Grab the information
    my $parser = new XML::DOM::Parser;
    my $doc = $parser->parsefile("$system{'datastore'}/alerts.xml");

    my $alertNode = "";

    foreach my $node ($doc->getElementsByTagName("Alert")) {
	
	if ($node->getAttribute("name") eq "networkport") {
	    $alertNode=$node;
	}
    }

    my $status = eval { $alertNode->getElementsByTagName("Status")->item(0)->getFirstChild->getData; } || "";
    my $email = eval { $alertNode->getElementsByTagName("Email")->item(0)->getFirstChild->getData; } || "";
    my $id = eval { $alertNode->getElementsByTagName("IDAttacker")->item(0)->getFirstChild->getData; } || "";

    $doc->dispose();

    my $netparser = new XML::DOM::Parser;
    my $netdoc = $netparser->parsefile("$system{'datastore'}/network.xml");
    
    my $telnet = 1;
    my $ftp = 1;
    my $ssh = 1;
    my $pop3 = 1;

    if ($status eq "Active") {

	print STDERR "Alert is active!\n";

	foreach my $interface ($netdoc->getElementsByTagName("Interface")) {

	    my $thistelnet = eval { $interface->getElementsByTagName("Telnet")->item(0)->getFirstChild->getData(); } || "No";
	    if ($thistelnet eq "No") {
		$telnet = 0;
	    }
	    my $thisftp = eval { $interface->getElementsByTagName("FTP")->item(0)->getFirstChild->getData(); } || "No";
	    if ($thisftp eq "No") {
		$ftp = 0;
	    }
	    my $thisssh = eval { $interface->getElementsByTagName("SSH")->item(0)->getFirstChild->getData(); } || "No";
	    if ($thisssh eq "No") {
		$ssh = 0;
	    }
	    my $thispop3 = eval { $interface->getElementsByTagName("POP3")->item(0)->getFirstChild->getData(); } || "No";
	    if ($thispop3 eq "No") {
		$pop3 = 0;
	    }
	}
	
    }

    $netdoc->dispose();

    print STDERR "Pop3 is $pop3\n";

    my $hostname = `/bin/hostname`;

    my $message = message::throwMessage("NETPORT_INFO", $email, $hostname, $hostname);

    # Write out the hosts.deny file
    if (open(DENYFILE, "</etc/hosts.deny")) {

	print STDERR "Read in host.deny\n";

	my @lines = <DENYFILE>;

	close(DENYFILE);

	open (DENYFILE, ">/etc/hosts.deny");

	print STDERR "Writing out hosts.deny\n";

	foreach my $line (@lines) {

	    if ($line =~ m/^popper : ALL :.*: DENY/) {

		print STDERR "Found popper line!\n";

		if ($pop3) {
		    next;
		}
		else {
		    if ($id eq "Yes") {
			print DENYFILE "popper : ALL : spawn (echo \"$message\" | /bin/mail -s \"ALERT - Network Port Access\" $email; /usr/sbin/safe_finger -l @%h | /bin/mail -s \"ALERT - Network Port Access - Attacker ID\" $email) &: DENY\n";
		    }
		    else {
			print DENYFILE "popper : ALL : spawn (echo \"$message\" | /bin/mail -s \"ALERT - Network Port Access\" $email) &: DENY\n";
		    }
		}
	    }
	    else {
		print DENYFILE $line;
	    }
	}

	close(DENYFILE);
    }

    return(1);

}

# A sub-routine that adds a user to the OS group
sub addUserToOSGroup {

    my $user = shift || return(0);
    my $group = shift || return(0);

#    print STDERR "                   <<<<<<<<<<<<<< Called addusertoosgroup \"$user\" \"$group\">>>>>>>>>>>>>>>>>\n";

    my $command = "/usr/bin/groups $user";
#    print STDERR "Checking users = $command\n";
    my $result = `$command`;

    my @pieces = split(/:/,$result);
    my @groups = split(/ /,$pieces[1]);
    push(@groups, $group);

    my $newgrouplist = "";

    foreach my $grp (@groups) {
	$grp =~ s/\n//;
	if ($grp ne "") {
	    if ($newgrouplist eq "") {
		$newgrouplist = "$grp";
	    }
	    else {
		$newgrouplist = $newgrouplist . ",$grp";
	    }
	}
    }

    # Step 2: Add them to every group and this one
    $command = "/usr/sbin/usermod -G $newgrouplist $user";
#    print STDERR " - ($group) Command = $command\n";
    $result = `$command`;
#    print STDERR "Result of addUserToOSGroup = $result\n";

}

# A sub-rotine that removes a user from the OS group
sub delUserFromOSGroup {

    my $user = shift || return(0);
    my $group = shift || return(0);

#    print STDERR "                   <<<<<<<<<<<<<< Called DELuserfromosgroup \"$user\" \"$group\">>>>>>>>>>>>>>>>>\n";

    my $command = "/usr/bin/groups $user";
    my $result = `$command`;

    $result =~ s/\n//g;
    my @pieces = split(/:/,$result);
    my @groups = split(/ /,$pieces[1]);

    # Remove the group
    my $newgrouplist = "";
    foreach my $thisGroup (@groups) {
#	print STDERR " &&& thisGroup = \"$thisGroup\" and group = \"$group\"\n";
	if ($thisGroup ne $group && $thisGroup ne "") {
	    $newgrouplist = $newgrouplist . "$thisGroup,";
	}
    }

    $newgrouplist = $newgrouplist . "$user";
    $newgrouplist =~ s/\n//g;

    # Step 2: Add them to every group and this one
    $command = "/usr/sbin/usermod -G $newgrouplist $user";
#    print STDERR " - ($group) Command = $command\n";
    $result = `$command`;
#    print STDERR "Result of delUserToOSGroup = $result\n";

}

# A sub-routine that upgrades the front page extension sites
sub updateVirtualSitesFPConf {

    my %system = config::system();

    # Make Front Page happy
    if (open(FPFILE, ">/etc/httpd/conf/fphttpd.conf")) {

	my $command = "/bin/cat $system{'httpd.conf'}";
	my $result = `$command`;
	
	print FPFILE "$result\n";
	
	$command = "/bin/cat /etc/httpd/conf/vhosts.conf";
	$result = `$command`;
	
	print FPFILE "$result\n";
	
	close(FPFILE);
	
	$command = "/bin/chmod o+r /etc/httpd/conf/fphttpd.conf";
	$result = `$command`;
	
    }

    my $filelist = `/bin/ls -a /usr/local/frontpage/*.cnf`;
    
    my @files = split("\n", $filelist);
    
    my @lines = ();
    
    foreach my $file (@files) {
	
	my $sitename = $file;
	
	$sitename =~ s/\/usr\/local\/frontpage\///;
	my @values = split(":", $sitename);
	$sitename = $values[0];
	
      custom::updateVirtualSite($sitename);
	
	# Read in the existing config file
	if (open(FPFILE, "<$file")) {
	    
	@lines = <FPFILE>;
	
	close(FPFILE);
    }

	if (open(FPFILE, ">$file")) {
	    
	    foreach my $line (@lines) {
		
		if ($line =~ m/serverconfig:/) {
		    $line = "serverconfig:/etc/httpd/conf/fphttpd.conf\n";
		}
		
		print FPFILE "$line";
		
	    }
	    
	    close(FPFILE);
	}
	
    }
    
}

sub setConfigStatus {

    my @lines = ();

    # Read the lines in

    if (open(FILE, "</var/www/html/index.html")) {

        @lines = <FILE>;
	close(FILE);
    }

    if (open(FILE, ">/var/www/html/index.html")) {
	
	foreach my $line(@lines) {
	    
	    $line =~ s/<META NAME="cpqstatus" CONTENT="1">//g;
	    print FILE $line;
	}

	close(FILE);
    }
}
	    
	    
1;

